@@ -1,7 +1,8 @@
 #' Methods for `glm_weightit()` objects
 #' @name anova.glm_weightit
 #'
-#' @description `anova()` is used to compare nested models fit with
+#' @description
+#' `anova()` is used to compare nested models fit with
 #' `glm_weightit()`, `mutinom_weightit()`, `ordinal_weightit()`, or
 #' `coxph_weightit()` using a Wald test that incorporates uncertainty in
 #' estimating the weights (if any).
@@ -15,14 +16,15 @@
 #'   `"Wald"` is allowed.
 #' @param tolerance for the Wald test, the tolerance used to determine if models
 #'   are symbolically nested.
-#'
 #' @param \dots other arguments passed to the function used for computing the
 #'   parameter variance matrix, if supplied as a string or function, e.g.,
 #'   `cluster`, `R`, or `fwb.args`.
 #'
-#' @returns An object of class `"anova"` inheriting from class `"data.frame"`.
+#' @returns
+#' An object of class `"anova"` inheriting from class `"data.frame"`.
 #'
-#' @details `anova()` performs a Wald test to compare two fitted models. The
+#' @details
+#' `anova()` performs a Wald test to compare two fitted models. The
 #' models must be nested, but they don't have to be nested symbolically (i.e.,
 #' the names of the coefficients of the smaller model do not have to be a subset
 #' of the names of the coefficients of the larger model). The larger model must
@@ -31,7 +33,8 @@
 #' variance-covariance matrix of the coefficients of the smaller model is not
 #' used.
 #'
-#' @seealso [glm_weightit()] for the page documenting `glm_weightit()`,
+#' @seealso
+#' [glm_weightit()] for the page documenting `glm_weightit()`,
 #' `lm_weightit()`, `ordinal_weightit()`, `multinom_weightit()`, and
 #' `coxph_weightit()`. [anova.glm()] for model comparison of `glm` objects.
 #'
@@ -58,14 +61,15 @@
 #' # Using a bootstrapped variance matrix
 #' anova(fit1, fit2, vcov = "BS", R = 100)
 #'
-#' @examplesIf requireNamespace("splines", quietly = TRUE)
+#' @examplesIf rlang::is_installed("splines")
 #' # Model comparison between spline model and linear
 #' # model; note they are nested but not symbolically
 #' # nested
 #' fit_s <- glm_weightit(re78 ~ splines::ns(age, df =4),
-#'                       data = lalonde )
+#'                       data = lalonde)
 #'
-#' fit_l <- glm_weightit( re78 ~ age, data = lalonde )
+#' fit_l <- glm_weightit(re78 ~ age,
+#'                       data = lalonde)
 #'
 #' anova(fit_s, fit_l)
 
@@ -132,14 +136,14 @@ anova.glm_weightit <- function(object, object2, test = "Chisq",
 
   SSH <- drop(crossprod(value.hyp, solve(vcov.hyp, value.hyp)))
 
-  .title <- paste0("\n", underline("Wald test"))
+  .title <- paste0("\n", .ul("Wald test"))
   .topnote <- sprintf("Model 1: %s\nModel 2: %s\n",
                       deparse1(formula(object)),
                       deparse1(formula(object2)))
 
-  .varnote <- paste(italic(sprintf("Variance: %s\n",
-                                   .vcov_to_phrase(object[["vcov_type"]],
-                                                   is_not_null(attr(object, "cluster"))))))
+  .varnote <- .it(sprintf("Variance: %s\n",
+                          .vcov_to_phrase(object[["vcov_type"]],
+                                          is_not_null(.attr(object, "cluster")))))
 
   result <- make_df(c("Res.Df", "Df", test, sprintf("Pr(>%s)", test)),
                     c("1", "2"))
@@ -235,14 +239,14 @@ anova.ordinal_weightit <- function(object, object2, test = "Chisq",
 
   SSH <- drop(crossprod(value.hyp, solve(vcov.hyp, value.hyp)))
 
-  .title <- paste0("\n", underline("Wald test"))
+  .title <- paste0("\n", .ul("Wald test"))
   .topnote <- sprintf("Model 1: %s\nModel 2: %s\n",
                       deparse1(formula(object)),
                       deparse1(formula(object2)))
 
-  .varnote <- paste(italic(sprintf("Variance: %s\n",
-                                   .vcov_to_phrase(object$vcov_type,
-                                                   is_not_null(attr(object, "cluster"))))))
+  .varnote <- .it(sprintf("Variance: %s\n",
+                          .vcov_to_phrase(object$vcov_type,
+                                          is_not_null(.attr(object, "cluster")))))
 
   result <- make_df(c("Res.Df", "Df", test, sprintf("Pr(>%s)", test)),
                     c("1", "2"))
@@ -289,8 +293,8 @@ anova.multinom_weightit <- function(object, object2, test = "Chisq",
   chk::chk_string(method)
   method <- match_arg(method, "Wald")
 
-  if (!identical(attr(object, "vcov_type"), attr(object2, "vcov_type")) &&
-      !identical(attr(object2, "vcov_type"), "none")) {
+  if (!identical(.attr(object, "vcov_type"), .attr(object2, "vcov_type")) &&
+      !identical(.attr(object2, "vcov_type"), "none")) {
     .wrn("different `vcov` types detected for each model; using the `vcov` from the larger model")
   }
 
@@ -347,14 +351,14 @@ anova.multinom_weightit <- function(object, object2, test = "Chisq",
 
   SSH <- drop(crossprod(value.hyp, solve(vcov.hyp, value.hyp)))
 
-  .title <- paste0("\n", underline("Wald test"))
+  .title <- paste0("\n", .ul("Wald test"))
   .topnote <- sprintf("Model 1: %s\nModel 2: %s\n",
                       deparse1(formula(object)),
                       deparse1(formula(object2)))
 
-  .varnote <- paste(italic(sprintf("Variance: %s\n",
-                                   .vcov_to_phrase(object$vcov_type,
-                                                   is_not_null(attr(object, "cluster"))))))
+  .varnote <- .it(sprintf("Variance: %s\n",
+                          .vcov_to_phrase(object$vcov_type,
+                                          is_not_null(.attr(object, "cluster")))))
 
   result <- make_df(c("Res.Df", "Df", test, sprintf("Pr(>%s)", test)),
                     c("1", "2"))
