@@ -1,31 +1,44 @@
 #chk utilities
 
-pkg_caller_call <- function(start = 1) {
+#Note: this version seems to do better when used inside tryCatch()
+pkg_caller_call <- function() {
   pn <- utils::packageName()
   package.funs <- c(getNamespaceExports(pn),
-                    .getNamespaceInfo(asNamespace(pn), "S3methods")[, 3])
-  k <- start #skip checking pkg_caller_call()
-  e_max <- start
-  while (!is.null(e <- rlang::caller_call(k))) {
-    if (!is.null(n <- rlang::call_name(e)) &&
-        n %in% package.funs) e_max <- k
-    k <- k + 1
+                    .getNamespaceInfo(asNamespace(pn), "S3methods")[, 3L])
+
+  for (i in seq_len(sys.nframe())) {
+    e <- sys.call(i)
+
+    n <- rlang::call_name(e)
+
+    if (is_null(n)) {
+      next
+    }
+
+    if (n %in% package.funs) {
+      return(e)
+    }
   }
-  rlang::caller_call(e_max)
+
+  NULL
 }
 
 .err <- function(..., n = NULL, tidy = TRUE) {
   m <- chk::message_chk(..., n = n, tidy = tidy)
   rlang::abort(paste(strwrap(m), collapse = "\n"),
-               call = pkg_caller_call(start = 2))
+               call = pkg_caller_call())
 }
 .wrn <- function(..., n = NULL, tidy = TRUE, immediate = TRUE) {
-  if (immediate && isTRUE(all.equal(0, getOption("warn")))) {
-    op <- options(warn = 1)
-    on.exit(options(op))
-  }
   m <- chk::message_chk(..., n = n, tidy = tidy)
-  rlang::warn(paste(strwrap(m), collapse = "\n"))
+
+  if (immediate && isTRUE(all.equal(0, getOption("warn")))) {
+    rlang::with_options({
+      rlang::warn(paste(strwrap(m), collapse = "\n"))
+    }, warn = 1)
+  }
+  else {
+    rlang::warn(paste(strwrap(m), collapse = "\n"))
+  }
 }
 .msg <- function(..., n = NULL, tidy = TRUE) {
   m <- chk::message_chk(..., n = n, tidy = tidy)
@@ -39,4 +52,4 @@ pkg_caller_call <- function(start = 1) {
   if (is.null(x_name))
     x_name <- chk::deparse_backtick_chk((substitute(x)))
   chk::abort_chk(x_name, " must be an atomic, non-matrix vector", x = x)
-}
\ No newline at end of file
+}
