@@ -6,9 +6,8 @@
 #using retained units, but full covariate matrix is returned.
 transform_covariates <- function(formula = NULL, data = NULL, method = "mahalanobis", s.weights = NULL, var = NULL,
                                  discarded = NULL) {
-  X <- get.covs.matrix.for.dist(formula, data)
-
-  X <- check_X(X)
+  X <- get.covs.matrix.for.dist(formula, data) |>
+    check_X()
 
   #If all variables have no variance, use Euclidean to avoid errors
   #If some have no variance, removes those to avoid messing up distances
@@ -30,7 +29,7 @@ transform_covariates <- function(formula = NULL, data = NULL, method = "mahalano
 
   if (method == "mahalanobis") {
 
-    if (is.null(var)) {
+    if (is_null(var)) {
       X <- scale(X)
       #NOTE: optmatch and Rubin (1980) use pooled within-group covariance matrix
       var <- {
@@ -104,8 +103,8 @@ transform_covariates <- function(formula = NULL, data = NULL, method = "mahalano
 eucdist_internal <- function(X, treat = NULL) {
 
   if (is.null(treat)) {
-    if (is.null(dim(X))) {
-      d <- abs(outer(X, X, "-"))
+    if (length(dim(X)) <= 1L) {
+      d <- abs(outer(drop(X), drop(X), "-"))
       dimnames(d) <- list(names(X), names(X))
     }
     else {
@@ -120,7 +119,7 @@ eucdist_internal <- function(X, treat = NULL) {
   }
   else {
     treat_l <- as.logical(treat)
-    if (is.null(dim(X))) {
+    if (length(dim(X)) <= 1L) {
       d <- abs(outer(X[treat_l], X[!treat_l], "-"))
       dimnames(d) <- list(names(X)[treat_l], names(X)[!treat_l])
     }
@@ -165,7 +164,7 @@ get.covs.matrix.for.dist <- function(formula = NULL, data = NULL) {
                                            function(x) contrasts(x, contrasts = FALSE) / sqrt(2)))
 
   if (ncol(X) > 1L) {
-    .assign <- attr(X, "assign")[-1L]
+    .assign <- .attr(X, "assign")[-1L]
     X <- X[, -1L, drop = FALSE]
   }
   attr(X, "assign") <- .assign
@@ -176,16 +175,16 @@ get.covs.matrix.for.dist <- function(formula = NULL, data = NULL) {
 }
 
 check_X <- function(X) {
-  if (isTRUE(attr(X, "checked"))) {
+  if (isTRUE(.attr(X, "checked"))) {
     return(X)
   }
 
-  treat <- attr(X, "treat")
+  treat <- .attr(X, "treat")
 
   if (is.data.frame(X)) {
     X <- as.matrix(X)
   }
-  else if (is.numeric(X) && is.null(dim(X))) {
+  else if (is.numeric(X) && length(dim(X)) <= 1L) {
     X <- matrix(X, nrow = length(X),
                 dimnames = list(names(X), NULL))
   }
