@@ -1,24 +1,24 @@
 # Makes a phrase from vcov_type to be printed by print() and summary()
 .vcov_to_phrase <- function(vcov_type, cluster = FALSE) {
   switch(vcov_type,
-         "const" = "maximum likelihood",
-         "asympt" = {
+         const = "maximum likelihood",
+         asympt = {
            if (cluster) "HC0 cluster-robust (adjusted for estimation of weights)"
            else"HC0 robust (adjusted for estimation of weights)"
          },
-         "HC0" = {
+         HC0 = {
            if (cluster) "HC0 cluster-robust"
            else "HC0 robust"
          },
-         "BS" = {
+         BS = {
            if (cluster) "traditional cluster bootstrap"
            else "traditional bootstrap"
          },
-         "FWB" = {
+         FWB = {
            if (cluster) "fractional weighted cluster bootstrap"
            else "fractional weighted bootstrap"
          },
-         "none" = "none",
+         none = "none",
          "user-supplied")
 }
 
@@ -36,8 +36,8 @@
     chk::chk_is(weightit, "weightit")
 
     if (!m_est_supported ||
-        (is_null(attr(weightit, "Mparts", exact = TRUE)) &&
-         is_null(attr(weightit, "Mparts.list", exact = TRUE)))) {
+        (is_null(.attr(weightit, "Mparts")) &&
+         is_null(.attr(weightit, "Mparts.list")))) {
 
       if (is_null(vcov)) {
         return("HC0")
@@ -98,7 +98,7 @@
   if (is_null(vcov.)) {
     V <- stats::vcov(object)
     vcov_type <- object$vcov_type
-    cluster <- attr(object, "cluster")
+    cluster <- .attr(object, "cluster")
   }
   else if (is.function(vcov.)) {
     V <- vcov.(object, ...)
@@ -119,8 +119,8 @@
   else if (chk::vld_string(vcov.)) {
     V <- .vcov_glm_weightit.internal(object, vcov. = vcov., ...)
 
-    vcov_type <- attr(V, "vcov_type")
-    cluster <- attr(V, "cluster")
+    vcov_type <- .attr(V, "vcov_type")
+    cluster <- .attr(V, "cluster")
   }
   else {
     .err("`vcov` must be `NULL`, a string corresponding to a method of computing the parameter variance matrix, a function that computes a variance matrix, or a variance matrix itself")
@@ -144,7 +144,7 @@
 
     V <- stats::vcov(object)
     vcov_type <- object$vcov_type
-    cluster <- attr(object, "cluster")
+    cluster <- .attr(object, "cluster")
   }
   else if (is.function(vcov.)) {
     V <- vcov.(object, ...)
@@ -165,8 +165,8 @@
   else if (chk::vld_string(vcov.)) {
     V <- .vcov_glm_weightit.internal(object, vcov. = vcov., ...)
 
-    vcov_type <- attr(V, "vcov_type")
-    cluster <- attr(V, "cluster")
+    vcov_type <- .attr(V, "vcov_type")
+    cluster <- .attr(V, "cluster")
   }
   else {
     .err("`vcov` must be `NULL`, a string corresponding to a method of computing the parameter variance matrix, a function that computes a variance matrix, or a variance matrix itself")
@@ -194,10 +194,10 @@
 
   object$vcov_type <- {
     if (is_null(vcov)) "none"
-    else if_null_then(vcov_type, attr(vcov, "vcov_type"))
+    else if_null_then(vcov_type, .attr(vcov, "vcov_type"))
   }
 
-  attr(object, "cluster") <- attr(vcov, "cluster")
+  attr(object, "cluster") <- .attr(vcov, "cluster")
 
   object$vcov <- .modify_vcov_info(object[["vcov"]])
 
@@ -215,7 +215,7 @@
     if (is_not_null(object[["vcov"]])) {
       return(.modify_vcov_info(object[["vcov"]],
                                vcov_type = object[["vcov_type"]],
-                               cluster = attr(object, "cluster")))
+                               cluster = .attr(object, "cluster")))
     }
 
     if (!identical(object[["vcov_type"]], "none")) {
@@ -238,7 +238,7 @@
 
   cluster <- {
     if ("cluster" %in% ...names()) ...get("cluster")
-    else attr(object, "cluster")
+    else .attr(object, "cluster")
   }
 
   internal_model_call <- .build_internal_model_call(object, vcov = vcov.)
@@ -272,7 +272,7 @@
                                        ...) {
   d <- dim(x)
 
-  if (is_null(d) || length(d) != 2L) {
+  if (length(d) != 2L) {
     .err("'x' must be coefficient matrix/data frame")
   }
 
@@ -445,7 +445,7 @@
 
     if (signif.legend) {
       w <- getOption("width")
-      sleg <- attr(Signif, "legend")
+      sleg <- .attr(Signif, "legend")
 
       if (w < nchar(sleg)) {
         sleg <- strwrap(sleg, width = w - 2L, prefix = space(2L))
@@ -591,7 +591,8 @@
   }
 
   if (vcov == "none") {
-    return(.modify_vcov_info(matrix(nrow = 0L, ncol = 0L), vcov_type = "none", cluster = cluster))
+    return(.modify_vcov_info(matrix(nrow = 0L, ncol = 0L),
+                             vcov_type = "none", cluster = cluster))
   }
 
   bout <- fit[["coefficients"]]
@@ -641,18 +642,18 @@
   }
 
   if (is_null(W)) {
-    W <- rep.int(1, length(Y))
+    W <- rep_with(1, Y)
   }
 
   if (is_null(SW)) {
-    SW <- rep.int(1, length(Y))
+    SW <- rep_with(1, Y)
   }
 
-  offset <- if_null_then(fit[["offset"]], rep.int(0, length(Y)))
+  offset <- if_null_then(fit[["offset"]], rep_with(0, Y))
 
   if (any(aliased)) {
-    if (is_not_null(attr(fit[["qr"]][["qr"]], "aliased"))) {
-      Xout <- Xout[, !attr(fit[["qr"]][["qr"]], "aliased"), drop = FALSE]
+    if (is_not_null(.attr(fit[["qr"]][["qr"]], "aliased"))) {
+      Xout <- Xout[, !.attr(fit[["qr"]][["qr"]], "aliased"), drop = FALSE]
     }
     else {
       Xout <- make_full_rank(Xout, with.intercept = FALSE)
@@ -702,8 +703,8 @@
   }
 
   if (vcov == "FWB") {
-    R <- attr(vcov, "R")
-    fwb.args <- attr(vcov, "fwb.args")
+    R <- .attr(vcov, "R")
+    fwb.args <- .attr(vcov, "fwb.args")
 
     internal_model_call$x <- FALSE
     internal_model_call$y <- FALSE
@@ -712,6 +713,10 @@
     if (is_not_null(weightit)) {
       wcall <- weightit[["call"]]
       wenv <- weightit[["env"]]
+
+      if (deparse1(wcall[[1L]]) %nin% c("weightit", "weightitMSM")) {
+        .err("the supplied `weightit` object does not appear to be the result of a call to `weightit()` or `weightitMSM()`, so bootstrapping cannot be used")
+      }
     }
     else {
       weightit_boot <- list(weights = 1)
@@ -723,8 +728,10 @@
       if (is_not_null(weightit)) {
         wcall$s.weights <- SW * w
 
-        weightit_boot <- .eval_fit(wcall, envir = wenv,
-                                   warnings = c("some extreme weights were generated" = NA))
+        suppressMessages({
+          weightit_boot <- .eval_fit(wcall, envir = wenv,
+                                     warnings = c("some extreme weights were generated" = NA))
+        })
 
         internal_model_call$weights <- weightit_boot[["weights"]] * SW * w
       }
@@ -735,7 +742,7 @@
       fit_boot <- .eval_fit(internal_model_call, envir = genv,
                             warnings = c("non-integer" = NA))
 
-      fit_boot[["coefficients"]]
+      fit_boot[["coefficients"]][!aliased]
     }
 
     #Sham data.frame to get weight length
@@ -761,16 +768,22 @@
     }
   }
   else if (vcov == "BS") {
-    R <- attr(vcov, "R")
+    R <- .attr(vcov, "R")
 
     internal_model_call$x <- FALSE
     internal_model_call$y <- FALSE
     internal_model_call$model <- FALSE
 
-    genv <- environment(fit$formula)
+    genv <- environment(fit[["formula"]])
+
     if (is_not_null(weightit)) {
-      wcall <- weightit$call
-      wenv <- weightit$env
+      wcall <- weightit[["call"]]
+      wenv <- weightit[["env"]]
+
+      if (deparse1(wcall[[1L]]) %nin% c("weightit", "weightitMSM")) {
+        .err("the supplied `weightit` object does not appear to be the result of a call to `weightit()` or `weightitMSM()`, so bootstrapping cannot be used")
+      }
+
       data <- eval(wcall$data, wenv)
       if (is_null(data)) {
         .err(sprintf('a dataset must have been supplied to `data` in the original call to `%s()` to use `vcov = "BS"`',
@@ -791,8 +804,10 @@
         wcall$data <- data[ind, ]
         wcall$s.weights <- SW[ind]
 
-        weightit_boot <- .eval_fit(wcall, envir = wenv,
-                                   warnings = c("some extreme weights were generated" = NA))
+        suppressMessages({
+          weightit_boot <- .eval_fit(wcall, envir = wenv,
+                                     warnings = c("some extreme weights were generated" = NA))
+        })
 
         internal_model_call$weights <- weightit_boot$weights * SW[ind]
       }
@@ -805,12 +820,12 @@
       fit_boot <- .eval_fit(internal_model_call, envir = genv,
                             warnings = c("non-integer" = NA))
 
-      fit_boot$coefficients
+      fit_boot$coefficients[!aliased]
     }
 
     if (is_null(cluster)) {
       boot_out <- do.call("rbind", lapply(seq_len(R), function(i) {
-        ind <- sample(seq_row(data), nrow(data), replace = TRUE)
+        ind <- sample.int(nrow(data), replace = TRUE)
         bootfun(data, ind)
       }))
 
@@ -822,7 +837,7 @@
         cli <- split(seq_along(cluster[[i]]), cluster[[i]])
 
         boot_out <- do.call("rbind", lapply(seq_len(R), function(i) {
-          ind <- unlist(cli[sample(seq_along(cli), length(cli), replace = TRUE)])
+          ind <- unlist(cli[sample.int(length(cli), replace = TRUE)])
           bootfun(data, ind)
         }))
 
@@ -832,7 +847,7 @@
   }
   else if (inherits(fit, "coxph")) {
     if (is_null(cluster)) {
-      V <- fit$var
+      V <- fit$var[!aliased, !aliased, drop = FALSE]
     }
     else {
       V <- 0
@@ -843,14 +858,14 @@
                           collapse = cluster[[i]],
                           weighted = TRUE)
 
-        V <- V + sgn[i] * adj * crossprod(temp)
+        V <- V + sgn[i] * adj * crossprod(temp[, !aliased, drop = FALSE])
       }
     }
   }
   else {
     if (vcov == "asympt") {
-      Mparts <- attr(weightit, "Mparts", exact = TRUE)
-      Mparts.list <- attr(weightit, "Mparts.list", exact = TRUE)
+      Mparts <- .attr(weightit, "Mparts")
+      Mparts.list <- .attr(weightit, "Mparts.list")
     }
     else {
       Mparts <- Mparts.list <- NULL
@@ -960,7 +975,7 @@
       if (all(lengths(dw_dBtreat.list) > 0L)) {
         w.list <- c(lapply(seq_along(btreat.list), function(i) {
           wfun.list[[i]](btreat.list[[i]], Xtreat.list[[i]], A.list[[i]])
-        }), list(rep(1, length(A.list[[1L]]))))
+        }), list(rep_with(1, A.list[[1L]])))
 
         dw_dBtreat <- do.call("cbind", lapply(seq_along(btreat.list), function(i) {
           dw_dBtreat.list[[i]](btreat.list[[i]], Xtreat.list[[i]], A.list[[i]], SW) *
@@ -1015,7 +1030,7 @@
     fit$model[["(weights)"]] <- weightit[["weights"]] * weightit[["s.weights"]]
   }
 
-  fit$vcov_type <- attr(fit[["vcov"]], "vcov_type")
+  fit$vcov_type <- .attr(fit[["vcov"]], "vcov_type")
 
   fit$call <- model_call
 
@@ -1026,7 +1041,7 @@
   if (isFALSE(x)) fit$x <- NULL
   if (isFALSE(y)) fit$y <- NULL
 
-  attr(fit, "cluster") <- attr(fit[["vcov"]], "cluster")
+  attr(fit, "cluster") <- .attr(fit[["vcov"]], "cluster")
 
   fit[["vcov"]] <- .modify_vcov_info(fit[["vcov"]])
 
@@ -1054,6 +1069,7 @@
   },
   error = function(e) {
     .e <- conditionMessage(e)
+
     if (grepl("system is computationally singular", .e, fixed = TRUE) ||
         grepl("Lapack routine dgesv: system is exactly singular", .e, fixed = TRUE)) {
       if (model == "out") {
