@@ -1,9 +1,10 @@
 #' Entropy Balancing
 #' @name method_ebal
-#' @aliases method_ebal
+#' @aliases method_entropy
 #' @usage NULL
 #'
-#' @description This page explains the details of estimating weights using
+#' @description
+#' This page explains the details of estimating weights using
 #' entropy balancing by setting `method = "ebal"` in the call to [weightit()] or
 #' [weightitMSM()]. This method can be used with binary, multi-category, and
 #' continuous treatments.
@@ -36,8 +37,8 @@
 #'
 #' For longitudinal treatments, the weights are the product of the weights
 #' estimated at each time point. This method is not guaranteed to yield exact
-#' balance at each time point. NOTE: the use of entropy balancing with
-#' longitudinal treatments has not been validated!
+#' balance at each time point. **NOTE: the use of entropy balancing with
+#' longitudinal treatments has not been validated and should not be done!**
 #'
 #' ## Sampling Weights
 #'
@@ -58,39 +59,34 @@
 #' M-estimation is supported for all scenarios. See [glm_weightit()] and
 #' `vignette("estimating-effects")` for details.
 #'
-#' @section Additional Arguments: `moments` and `int` are accepted. See
-#'   [weightit()] for details.
+#' @section Additional Arguments:
 #'
 #' \describe{
-#'   \item{`base.weights`}{
-#'     A vector of base weights, one for each unit. These correspond to the base weights $q$ in Hainmueller (2012). The estimated weights minimize the Kullback entropy divergence from the base weights, defined as \eqn{\sum w \log(w/q)}, subject to exact balance constraints. These can be used to supply previously estimated weights so that the newly estimated weights retain the some of the properties of the original weights while ensuring the balance constraints are met. Sampling weights should not be passed to `base.weights` but can be included in a `weightit()` call that includes `s.weights`.
-#'   }
-#'   \item{`reltol`}{the relative tolerance for convergence of the optimization. Passed to the `control` argument of `optim()`. Default is `1e-10`.
+#'   \item{`base.weights`}{a vector of base weights, one for each unit. These correspond to the base weights $q$ in Hainmueller (2012). The estimated weights minimize the Kullback entropy divergence from the base weights, defined as \eqn{\sum w \log(w/q)}, subject to exact balance constraints. These can be used to supply previously estimated weights so that the newly estimated weights retain the some of the properties of the original weights while ensuring the balance constraints are met. Sampling weights should not be passed to `base.weights` but can be included in a `weightit()` call that includes `s.weights`.}
+#'   \item{`reltol`}{the relative tolerance for convergence of the optimization. Passed to the `control` argument of `optim()`. Default is `1e-10`.}
+#'   \item{`maxit`}{the maximum number of iterations for convergence of the optimization. Passed to the `control` argument of `optim()`. Default is 1000 for binary and multi-category treatments and 10000 for continuous and longitudinal treatments.}
+#'   \item{`solver`}{the solver to use to estimate the parameters. Allowable options include `"multiroot"` to use \pkgfun{rootSolve}{multiroot} and `"optim"` to use [stats::optim()]. `"multiroot"` is the default when \pkg{rootSolve} is installed, as it tends to be much faster and more accurate; otherwise, `"optim"` is the default and requires no dependencies. Regardless of `solver`, the output of `optim()` is returned when `include.obj = TRUE` (see below).}
+#'   \item{`moments`}{`integer`; the highest power of each covariate to be balanced. For example, if `moments = 3`, each covariate, its square, and its cube will be balanced. Can also be a named vector with a value for each covariate (e.g., `moments = c(x1 = 2, x2 = 4)`). Values greater than 1 for categorical covariates are ignored. Default is 1 to balance covariate means.
 #'     }
-#'     \item{`maxit`}{the maximum number of iterations for convergence of the optimization. Passed to the `control` argument of `optim()`. Default is 1000 for binary and multi-category treatments and 10000 for continuous and longitudinal treatments.
+#'     \item{`int`}{`logical`; whether first-order interactions of the covariates are to be balanced. Default is `FALSE`.
 #'     }
-#'     \item{`solver`}{the solver to use to estimate the parameters of the just-identified CBPS. Allowable options include `"multiroot"` to use \pkgfun{rootSolve}{multiroot} and `"optim"` to use [stats::optim()]. `"multiroot"` is the default when \pkg{rootSolve} is installed, as it tends to be much faster and more accurate; otherwise, `"optim"` is the default and requires no dependencies. Regardless of `solver`, the output of `optim()` is returned when `include.obj = TRUE` (see below). When `over = TRUE`, the parameter estimates of the just-identified CBPS are used as starting values for the over-identified CBPS.
+#'     \item{`quantile`}{a named list of quantiles (values between 0 and 1) for each continuous covariate, which are used to create additional variables that when balanced ensure balance on the corresponding quantile of the variable. For example, setting `quantile = list(x1 = c(.25, .5. , .75))` ensures the 25th, 50th, and 75th percentiles of `x1` in each treatment group will be balanced in the weighted sample. Can also be a single number (e.g., `.5`) or a vector (e.g., `c(.25, .5, .75)`) to request the same quantile(s) for all continuous covariates. Only allowed with binary and multi-category treatments.
 #'     }
-#'   \item{`quantile`}{
-#'     A named list of quantiles (values between 0 and 1) for each continuous covariate, which are used to create additional variables that when balanced ensure balance on the corresponding quantile of the variable. For example, setting `quantile = list(x1 = c(.25, .5. , .75))` ensures the 25th, 50th, and 75th percentiles of `x1` in each treatment group will be balanced in the weighted sample. Can also be a single number (e.g., `.5`) or an unnamed list of length 1 (e.g., `list(c(.25, .5, .75))`) to request the same quantile(s) for all continuous covariates, or a named vector (e.g., `c(x1 = .5, x2 = .75)` to request one quantile for each covariate. Only allowed with binary and multi-category treatments.
-#'   }
-#'   \item{`d.moments`}{
-#'     With continuous treatments, the number of moments of the treatment and covariate distributions that are constrained to be the same in the weighted sample as in the original sample. For example, setting `d.moments = 3` ensures that the mean, variance, and skew of the treatment and covariates are the same in the weighted sample as in the unweighted sample. `d.moments` should be greater than or equal to `moments` and will be automatically set accordingly if not (or if not specified). Vegetabile et al. (2021) recommend setting `d.moments = 3`, even if `moments` is less than 3. This argument corresponds to the tuning parameters $r$ and $s$ in Vegetabile et al. (2021) (which here are set to be equal). Ignored for binary and multi-category treatments.
-#'   }
+#'   \item{`d.moments`}{`integer`; with continuous treatments, the number of moments of the treatment and covariate distributions that are constrained to be the same in the weighted sample as in the original sample. For example, setting `d.moments = 3` ensures that the mean, variance, and skew of the treatment and covariates are the same in the weighted sample as in the unweighted sample. `d.moments` should be greater than or equal to `moments` and will be automatically set accordingly if not (or if not specified). Vegetabile et al. (2021) recommend setting `d.moments = 3`, even if `moments` is less than 3. This argument corresponds to the tuning parameters \eqn{r} and \eqn{s} in Vegetabile et al. (2021) (which here are set to be equal). Ignored for binary and multi-category treatments.}
 #' }
 #'
-#'   The `stabilize` argument is ignored; in the past it would reduce the
-#'   variability of the weights through an iterative process. If you want to
-#'   minimize the variance of the weights subject to balance constraints, use
-#'   `method = "optweight"`.
+#' The `stabilize` argument is ignored; in the past it would reduce the
+#' variability of the weights through an iterative process. If you want to
+#' minimize the variance of the weights subject to balance constraints, use
+#' `method = "optweight"`.
 #'
 #' @section Additional Outputs:
 #' \describe{
-#'   \item{`obj`}{When `include.obj = TRUE`, the output of the call to [optim()], which contains the dual variables and convergence information. For ATE fits or with multi-category treatments, a list of `optim()` outputs, one for each weighted group.
-#'   }
+#'   \item{`obj`}{When `include.obj = TRUE`, the output of the call to [optim()], which contains the dual variables and convergence information. For ATE fits or with multi-category treatments, a list of `optim()` outputs, one for each weighted group.}
 #' }
 #'
-#' @details Entropy balancing involves the specification of an optimization
+#' @details
+#' Entropy balancing involves the specification of an optimization
 #' problem, the solution to which is then used to compute the weights. The
 #' constraints of the primal optimization problem correspond to covariate
 #' balance on the means (for binary and multi-category treatments) or
@@ -98,7 +94,9 @@
 #' the weights, and that the weights sum to a certain value. It turns out that
 #' the dual optimization problem is much easier to solve because it is over only
 #' as many variables as there are balance constraints rather than over the
-#' weights for each unit and it is unconstrained. Zhao and Percival (2017) found
+#' weights for each unit, and it is unconstrained.
+#'
+#' Zhao and Percival (2017) found
 #' that entropy balancing for the ATT of a binary treatment actually involves
 #' the estimation of the coefficients of a logistic regression propensity score
 #' model but using a specialized loss function different from that optimized
@@ -115,32 +113,25 @@
 #' [method_ipt] and [method_cbps] for inverse probability tilting and CBPS,
 #' which work similarly.
 #'
-#' @references ## Binary Treatments
+#' @references
 #'
-#' ### `estimand = "ATT"` Hainmueller, J. (2012). Entropy Balancing for Causal
-#' Effects: A Multivariate Reweighting Method to Produce Balanced Samples in
-#' Observational Studies. *Political Analysis*, 20(1), 25–46.
-#' \doi{10.1093/pan/mpr025}
+#' ## Binary Treatments
 #'
-#' Zhao, Q., & Percival, D. (2017). Entropy balancing is doubly robust. *Journal
-#' of Causal Inference*, 5(1). \doi{10.1515/jci-2016-0010}
+#' ### `estimand = "ATT"`
+#'
+#' Hainmueller, J. (2012). Entropy Balancing for Causal Effects: A Multivariate Reweighting Method to Produce Balanced Samples in Observational Studies. *Political Analysis*, 20(1), 25–46. \doi{10.1093/pan/mpr025}
+#'
+#' Zhao, Q., & Percival, D. (2017). Entropy balancing is doubly robust. *Journal of Causal Inference*, 5(1). \doi{10.1515/jci-2016-0010}
 #'
 #' ### `estimand = "ATE"`
 #'
-#' Källberg, D., & Waernbaum, I. (2023). Large Sample Properties of Entropy
-#' Balancing Estimators of Average Causal Effects. *Econometrics and
-#' Statistics*. \doi{10.1016/j.ecosta.2023.11.004}
+#' Källberg, D., & Waernbaum, I. (2023). Large Sample Properties of Entropy Balancing Estimators of Average Causal Effects. *Econometrics and Statistics*. \doi{10.1016/j.ecosta.2023.11.004}
 #'
 #' ## Continuous Treatments
 #'
-#' Tübbicke, S. (2022). Entropy Balancing for Continuous Treatments. *Journal of
-#' Econometric Methods*, 11(1), 71–89. \doi{10.1515/jem-2021-0002}
+#' Tübbicke, S. (2022). Entropy Balancing for Continuous Treatments. *Journal of Econometric Methods*, 11(1), 71–89. \doi{10.1515/jem-2021-0002}
 #'
-#' Vegetabile, B. G., Griffin, B. A., Coffman, D. L., Cefalu, M., Robbins, M.
-#' W., & McCaffrey, D. F. (2021). Nonparametric estimation of population average
-#' dose-response curves using entropy balancing weights for continuous
-#' exposures. *Health Services and Outcomes Research Methodology*, 21(1),
-#' 69–110. \doi{10.1007/s10742-020-00236-2}
+#' Vegetabile, B. G., Griffin, B. A., Coffman, D. L., Cefalu, M., Robbins, M. W., & McCaffrey, D. F. (2021). Nonparametric estimation of population average dose-response curves using entropy balancing weights for continuous exposures. *Health Services and Outcomes Research Methodology*, 21(1), 69–110. \doi{10.1007/s10742-020-00236-2}
 #'
 #' @examples
 #' data("lalonde", package = "cobalt")
@@ -149,14 +140,18 @@
 #' (W1 <- weightit(treat ~ age + educ + married +
 #'                   nodegree + re74, data = lalonde,
 #'                 method = "ebal", estimand = "ATT"))
+#'
 #' summary(W1)
+#'
 #' cobalt::bal.tab(W1)
 #'
 #' #Balancing covariates with respect to race (multi-category)
 #' (W2 <- weightit(race ~ age + educ + married +
 #'                   nodegree + re74, data = lalonde,
 #'                 method = "ebal", estimand = "ATE"))
+#'
 #' summary(W2)
+#'
 #' cobalt::bal.tab(W2)
 #'
 #' #Balancing covariates and squares with respect to
@@ -166,12 +161,15 @@
 #'                   nodegree + re74, data = lalonde,
 #'                 method = "ebal", moments = 2,
 #'                 d.moments = 3))
+#'
 #' summary(W3)
-#' cobalt::bal.tab(W3)
+#'
+#' cobalt::bal.tab(W3, poly = 2,
+#'                 stats = c("c", "m"))
 NULL
 
 weightit2ebal <- function(covs, treat, s.weights, subset, estimand, focal,
-                          stabilize, missing, moments, int, verbose, ...) {
+                          stabilize, missing, verbose, ...) {
 
   covs <- covs[subset, , drop = FALSE]
   treat <- factor(treat[subset])
@@ -183,9 +181,12 @@ weightit2ebal <- function(covs, treat, s.weights, subset, estimand, focal,
     covs <- add_missing_indicators(covs)
   }
 
-  covs <- cbind(.int_poly_f(covs, poly = moments, int = int, center = TRUE),
-                .quantile_f(covs, qu = ...get("quantile"), s.weights = s.weights,
-                            focal = focal, treat = treat))
+  covs <- .apply_moments_int_quantile(covs,
+                                      moments = ...get("moments"),
+                                      int = ...get("int"),
+                                      quantile = ...get("quantile"),
+                                      s.weights = s.weights, focal = focal,
+                                      treat = treat)
 
   for (i in seq_col(covs)) {
     covs[, i] <- .make_closer_to_1(covs[, i])
@@ -196,7 +197,7 @@ weightit2ebal <- function(covs, treat, s.weights, subset, estimand, focal,
 
   bw <- if_null_then(...get("base.weights"),
                      ...get("base.weight"),
-                     rep.int(1, length(treat)))
+                     rep_with(1, treat))
 
   if (!is.numeric(bw) || length(bw) != length(treat)) {
     .err("the argument to `base.weight` must be a numeric vector with length equal to the number of units")
@@ -208,13 +209,13 @@ weightit2ebal <- function(covs, treat, s.weights, subset, estimand, focal,
   maxit <- ...get("maxit", 1e4L)
   chk::chk_count(maxit)
 
-  solver <- ...get("solver", NULL)
+  solver <- ...get("solver")
   if (is_null(solver)) {
-    if (requireNamespace("rootSolve", quietly = TRUE)) {
-      solver <- "multiroot"
-    }
-    else {
-      solver <- "optim"
+    solver <- {
+      if (rlang::is_installed("rootSolve"))
+        "multiroot"
+      else
+        "optim"
     }
   }
   else {
@@ -246,16 +247,20 @@ weightit2ebal <- function(covs, treat, s.weights, subset, estimand, focal,
 
     if (solver == "multiroot") {
       out <- suppressWarnings({
-        try(rootSolve::multiroot(f = gradient.EB,
-                                 start = coef_start,
-                                 S = s.weights_t, C = C, Q = Q,
-                                 rtol = reltol,
-                                 atol = reltol,
-                                 ctol = reltol),
-            silent = TRUE)
+        try(verbosely({
+          rootSolve::multiroot(f = gradient.EB,
+                               start = coef_start,
+                               S = s.weights_t, C = C, Q = Q,
+                               rtol = reltol,
+                               atol = reltol,
+                               ctol = reltol)
+        }, verbose = FALSE), silent = TRUE)
       })
 
-      if (!null_or_error(out) && out$estim.precis < 1e-5) {
+      if (!null_or_error(out) && utils::hasName(out, "root") &&
+          utils::hasName(out, "estim.precis") &&
+          chk::vld_number(out[["estim.precis"]]) &&
+          out[["estim.precis"]] < 1e-5) {
         coef_start <- out$root
       }
     }
@@ -289,7 +294,7 @@ weightit2ebal <- function(covs, treat, s.weights, subset, estimand, focal,
          opt.out = opt.out)
   }
 
-  w <- rep.int(1, length(treat))
+  w <- rep_with(1, treat)
   sw0 <- check_if_zero(s.weights)
 
   if (estimand == "ATE") {
@@ -343,7 +348,7 @@ weightit2ebal <- function(covs, treat, s.weights, subset, estimand, focal,
       }), groups_to_weight)
 
       sw0 <- check_if_zero(s.weights)
-      w <- rep.int(1, length(A))
+      w <- rep_with(1, A)
 
       for (i in groups_to_weight) {
         in_i <- which(A == i & !sw0)
@@ -388,7 +393,7 @@ weightit2ebal <- function(covs, treat, s.weights, subset, estimand, focal,
 
       sw0 <- check_if_zero(SW)
 
-      H <- matrix(0, nrow = length(Btreat), ncol = length(Btreat))
+      H <- sq_matrix(0, n = length(Btreat))
 
       for (i in groups_to_weight) {
         in_i <- which(A == i & !sw0)
@@ -412,7 +417,7 @@ weightit2ebal <- function(covs, treat, s.weights, subset, estimand, focal,
 
 weightit2ebal.multi <- weightit2ebal
 
-weightit2ebal.cont <- function(covs, treat, s.weights, subset, missing, moments, int, verbose, ...) {
+weightit2ebal.cont <- function(covs, treat, s.weights, subset, missing, verbose, ...) {
 
   covs <- covs[subset, , drop = FALSE]
   s.weights <- s.weights[subset]
@@ -423,12 +428,21 @@ weightit2ebal.cont <- function(covs, treat, s.weights, subset, missing, moments,
     covs <- add_missing_indicators(covs)
   }
 
-  bw <- if_null_then(...get("base.weights"),
-                     ...get("base.weight"),
-                     rep.int(1, length(treat)))
+  for (i in c("b.weights", "base.weights", "base.weight")) {
+    bw <- ...get(i)
 
-  if (!is.numeric(bw) || length(bw) != length(treat)) {
-    .err("the argument to `base.weight` must be a numeric vector with length equal to the number of units")
+    if (is_not_null(bw)) {
+      if (!is.numeric(bw) || length(bw) != length(treat)) {
+        .err(sprintf("the argument to `%s` must be a numeric vector with length equal to the number of units",
+                     i))
+      }
+
+      break
+    }
+  }
+
+  if (is_null(bw)) {
+    bw <- rep_with(1, treat)
   }
 
   treat <- treat[subset]
@@ -445,7 +459,7 @@ weightit2ebal.cont <- function(covs, treat, s.weights, subset, missing, moments,
 
   solver <- ...get("solver", NULL)
   if (is_null(solver)) {
-    if (requireNamespace("rootSolve", quietly = TRUE)) {
+    if (rlang::is_installed("rootSolve")) {
       solver <- "multiroot"
     }
     else {
@@ -461,25 +475,29 @@ weightit2ebal.cont <- function(covs, treat, s.weights, subset, missing, moments,
     rlang::check_installed("rootSolve")
   }
 
-  d.moments <- max(...get("d.moments", 1L), moments)
+  moments <- ...get("moments", 1L)
+
+  d.moments <- ...get("d.moments", 1L)
   chk::chk_count(d.moments)
 
   treat <- .make_closer_to_1(treat)
 
   t.mat <- matrix(treat, ncol = 1L, dimnames = list(NULL, "treat"))
-  t.mat <- .int_poly_f(t.mat, poly = d.moments)
+  t.mat <- .apply_moments_int_quantile(t.mat, moments = d.moments)
 
-  t.mat <- center(t.mat, cobalt::col_w_mean(t.mat, s.weights))
+  t.mat <- center_w(t.mat, s.weights)
 
-  bal.covs <- .int_poly_f(covs, poly = moments, int = int, center = TRUE)
+  bal.covs <- .apply_moments_int_quantile(covs,
+                                          moments = moments,
+                                          int = ...get("int"))
 
   for (i in seq_col(bal.covs)) {
     bal.covs[, i] <- .make_closer_to_1(bal.covs[, i])
   }
 
-  bal.covs <- center(bal.covs, cobalt::col_w_mean(bal.covs, s.weights))
+  bal.covs <- center_w(bal.covs, s.weights)
 
-  if (d.moments == moments) {
+  if (all(d.moments <= moments)) {
     C <- cbind(t.mat,
                bal.covs,
                t.mat[, 1L] * bal.covs)
@@ -489,13 +507,15 @@ weightit2ebal.cont <- function(covs, treat, s.weights, subset, missing, moments,
                      colnames(bal.covs))
   }
   else {
-    d.covs <- .int_poly_f(covs, poly = d.moments, int = int, center = TRUE)
+    d.covs <- .apply_moments_int_quantile(covs,
+                                          moments = pmax(d.moments, moments),
+                                          int = ...get("int"))
 
     for (i in seq_col(d.covs)) {
       d.covs[, i] <- .make_closer_to_1(d.covs[, i])
     }
 
-    d.covs <- center(d.covs, cobalt::col_w_mean(d.covs, s.weights))
+    d.covs <- center_w(d.covs, s.weights)
 
     C <- cbind(t.mat,
                d.covs,
@@ -529,15 +549,18 @@ weightit2ebal.cont <- function(covs, treat, s.weights, subset, missing, moments,
 
     if (solver == "multiroot") {
       out <- suppressWarnings({
-        try(rootSolve::multiroot(f = gradient.EB,
-                                 start = coef_start,
-                                 S = s.weights, C = C, Q = Q,
-                                 maxiter = 20),
-            silent = TRUE)
+        try(verbosely({
+          rootSolve::multiroot(f = gradient.EB,
+                               start = coef_start,
+                               S = s.weights, C = C, Q = Q,
+                               maxiter = 20)
+        }, verbose = FALSE), silent = TRUE)
       })
 
-      if (!null_or_error(out) && is.finite(out$estim.precis) &&
-          out$estim.precis < 1e-5) {
+      if (!null_or_error(out) && utils::hasName(out, "root") &&
+          utils::hasName(out, "estim.precis") &&
+          chk::vld_number(out[["estim.precis"]]) &&
+          out[["estim.precis"]] < 1e-5) {
         coef_start <- out$root
       }
     }
@@ -571,7 +594,7 @@ weightit2ebal.cont <- function(covs, treat, s.weights, subset, missing, moments,
          opt.out = opt.out)
   }
 
-  w <- rep.int(1, length(treat))
+  w <- rep_with(1, treat)
   sw0 <- check_if_zero(s.weights)
 
   verbosely({
@@ -591,7 +614,7 @@ weightit2ebal.cont <- function(covs, treat, s.weights, subset, missing, moments,
     },
     wfun = function(Btreat, Xtreat, A) {
       sw0 <- check_if_zero(s.weights)
-      w <- rep.int(1, length(A))
+      w <- rep_with(1, A)
 
       C <- Xtreat[!sw0, , drop = FALSE]
       n <- nrow(C)
@@ -627,3 +650,221 @@ weightit2ebal.cont <- function(covs, treat, s.weights, subset, missing, moments,
   list(w = w, fit.obj = fit$opt.out,
        Mparts = Mparts)
 }
+
+# .weightit2ebal.cont <- function(covs, treat, s.weights, subset, missing, moments, int, verbose, ...) {
+#
+#   covs <- covs[subset, , drop = FALSE]
+#   s.weights <- s.weights[subset]
+#
+#   missing <- .process_missing2(missing, covs)
+#
+#   if (missing == "ind") {
+#     covs <- add_missing_indicators(covs)
+#   }
+#
+#   bw <- if_null_then(...get("base.weights"),
+#                      ...get("base.weight"),
+#                      rep.int(1, length(treat)))
+#
+#   if (!is.numeric(bw) || length(bw) != length(treat)) {
+#     .err("the argument to `base.weight` must be a numeric vector with length equal to the number of units")
+#   }
+#
+#   treat <- treat[subset]
+#
+#   bw <- bw[subset]
+#
+#   s.weights <- s.weights / mean_fast(s.weights)
+#
+#   reltol <- ...get("reltol", 1e-10)
+#   chk::chk_number(reltol)
+#
+#   maxit <- ...get("maxit", 1e4)
+#   chk::chk_count(maxit)
+#
+#   solver <- ...get("solver", NULL)
+#   if (is_null(solver)) {
+#     if (rlang::is_installed("rootSolve")) {
+#       solver <- "multiroot"
+#     }
+#     else {
+#       solver <- "optim"
+#     }
+#   }
+#   else {
+#     chk::chk_string(solver)
+#     solver <- match_arg(solver, c("optim", "multiroot"))
+#   }
+#
+#   if (solver == "multiroot") {
+#     rlang::check_installed("rootSolve")
+#   }
+#
+#   d.moments <- max(...get("d.moments", 1L), moments)
+#   chk::chk_count(d.moments)
+#
+#   treat <- .make_closer_to_1(treat)
+#
+#   t.mat <- matrix(treat, ncol = 1L, dimnames = list(NULL, "treat"))
+#   t.mat <- .int_poly_f(t.mat, poly = d.moments)
+#
+#   t.mat <- scale(t.mat,
+#                  scale = cobalt::col_w_sd(t.mat, s.weights),
+#                  center = cobalt::col_w_mean(t.mat, s.weights))
+#
+#   bal.covs <- .int_poly_f(covs, poly = moments, int = int, center = TRUE)
+#
+#   bal.covs <- scale(bal.covs,
+#                     scale = cobalt::col_w_sd(bal.covs, s.weights),
+#                     center = cobalt::col_w_mean(bal.covs, s.weights))
+#
+#   if (d.moments == moments) {
+#     C <- cbind(t.mat,
+#                bal.covs,
+#                t.mat[, 1L] * bal.covs)
+#
+#     colnames(C) <- c(paste(colnames(t.mat), "(mean)"),
+#                      paste(colnames(bal.covs), "(mean)"),
+#                      colnames(bal.covs))
+#   }
+#   else {
+#     d.covs <- .int_poly_f(covs, poly = d.moments, int = int, center = TRUE)
+#
+#     d.covs <- scale(d.covs,
+#                     scale = cobalt::col_w_sd(d.covs, s.weights),
+#                     center = cobalt::col_w_mean(d.covs, s.weights))
+#
+#     C <- cbind(t.mat,
+#                d.covs,
+#                t.mat[, 1L] * bal.covs)
+#
+#     colnames(C) <- c(paste(colnames(t.mat), "(mean)"),
+#                      paste(colnames(d.covs), "(mean)"),
+#                      colnames(bal.covs))
+#   }
+#
+#   colinear.covs.to.remove <- setdiff(colnames(C), colnames(make_full_rank(C)))
+#   C <- C[, colnames(C) %nin% colinear.covs.to.remove, drop = FALSE]
+#
+#   eb <- function(C, s.weights, Q) {
+#     n <- nrow(C)
+#
+#     W <- function(Z, Q, C) {
+#       Q * exp(drop(C %*% Z))
+#     }
+#
+#     objective.EB <- function(Z, S, Q, C) {
+#       sum(S * W(Z, Q, C)) / n - Z[1L]
+#     }
+#
+#     gradient.EB <- function(Z, S, Q, C) {
+#       w <- S * W(Z, Q, C)
+#       drop(w %*% C) / n - c(1, rep(0, ncol(C) - 1))
+#     }
+#
+#     coef_start <- rep.int(0, ncol(C))
+#
+#     if (solver == "multiroot") {
+#       out <- suppressWarnings({
+#         try(verbosely({
+#         rootSolve::multiroot(f = gradient.EB,
+#                                  start = coef_start,
+#                                  S = s.weights, C = C, Q = Q,
+#                                  maxiter = 20)
+#             }, verbose = FALSE), silent = TRUE)
+#       })
+#
+#       if (!null_or_error(out) && utils::hasName(out, "root") &&
+#           utils::hasName(out, "estim.precis") &&
+#           chk::vld_number(out[["estim.precis"]]) &&
+#           out[["estim.precis"]] < 1e-5) {
+#         coef_start <- out$root
+#       }
+#     }
+#
+#     opt.out <- optim(par = coef_start,
+#                      fn = objective.EB,
+#                      gr = gradient.EB,
+#                      method = "BFGS",
+#                      control = list(trace = 1,
+#                                     reltol = reltol,
+#                                     maxit = maxit),
+#                      S = s.weights, Q = Q, C = C,
+#                      hessian = TRUE)
+#
+#     w <- W(opt.out$par, Q, C)
+#     opt.out$gradient <- gradient.EB(opt.out$par, s.weights, Q, C)
+#
+#     if (opt.out$convergence != 0) {
+#       .wrn("the optimization failed to converge in the alotted number of iterations. Try increasing `maxit`")
+#     }
+#     else if (any(abs(opt.out$gradient) > 1e-3)) {
+#       .wrn("the estimated weights do not balance the covariates, indicating the optimization arrived at a degenerate solution. Try decreasing the number of variables supplied to the optimization")
+#     }
+#
+#     # if (sum(w) > n * .Machine$double.eps) {
+#     #   w <- w * n / sum(w)
+#     # }
+#
+#     list(Z = setNames(opt.out$par, colnames(C)),
+#          w = w,
+#          opt.out = opt.out)
+#   }
+#
+#   w <- rep.int(1, length(treat))
+#   sw0 <- check_if_zero(s.weights)
+#
+#   verbosely({
+#     fit <- eb(cbind(1, C[!sw0, , drop = FALSE]), s.weights[!sw0], bw[!sw0])
+#   }, verbose = verbose)
+#
+#   w[!sw0] <- fit$w
+#
+#   Mparts <- list(
+#     psi_treat = function(Btreat, Xtreat, A, SW) {
+#       sw0 <- check_if_zero(SW)
+#
+#       C <- Xtreat[!sw0, , drop = FALSE]
+#       w <- SW[!sw0] * bw[!sw0] * exp(drop(C %*% Btreat))
+#
+#       sweep(w * C, 2L, c(1, rep(0, ncol(Xtreat) - 1L)))
+#     },
+#     wfun = function(Btreat, Xtreat, A) {
+#       sw0 <- check_if_zero(s.weights)
+#       w <- rep.int(1, length(A))
+#
+#       C <- Xtreat[!sw0, , drop = FALSE]
+#       n <- nrow(C)
+#       w[!sw0] <- bw[!sw0] * exp(drop(C %*% Btreat))
+#
+#       # if (sum(w[!sw0]) > n * .Machine$double.eps) {
+#       #   w[!sw0] <- w[!sw0] * n / sum(w[!sw0])
+#       # }
+#
+#       w
+#     },
+#     dwdB = function(Btreat, Xtreat, A, SW) {
+#       sw0 <- check_if_zero(SW)
+#
+#       C <- Xtreat[!sw0, , drop = FALSE]
+#       w <- bw[!sw0] * exp(drop(C %*% Btreat))
+#
+#       sweep(C, 2L, colSums(w * C), "-")
+#     },
+#     hess_treat = function(Btreat, Xtreat, A, SW) {
+#       sw0 <- check_if_zero(SW)
+#
+#       C <- Xtreat[!sw0, , drop = FALSE]
+#       w <- SW[!sw0] * bw[!sw0] * exp(drop(C %*% Btreat))
+#
+#       crossprod(sweep(w * C, 2L, c(1, rep(0, ncol(Xtreat) - 1L))), C)
+#       # crossprod(C * w / sum(w), C)
+#     },
+#     Xtreat = cbind(1, C),
+#     A = treat,
+#     btreat = fit$Z
+#   )
+#
+#   list(w = w, fit.obj = fit$opt.out,
+#        Mparts = Mparts)
+# }
