@@ -1,9 +1,9 @@
 #' Energy Balancing
 #' @name method_energy
-#' @aliases method_energy
 #' @usage NULL
 #'
-#' @description This page explains the details of estimating weights using
+#' @description
+#' This page explains the details of estimating weights using
 #' energy balancing by setting `method = "energy"` in the call to [weightit()]
 #' or [weightitMSM()]. This method can be used with binary, multi-category, and
 #' continuous treatments.
@@ -11,7 +11,7 @@
 #' In general, this method relies on estimating weights by minimizing an energy
 #' statistic related to covariate balance. For binary and multi-category
 #' treatments, this is the energy distance, which is a multivariate distance
-#' between distributions, between treatment groups. For continuous treatments,
+#' between distributions, between the treatment groups. For continuous treatments,
 #' this is the sum of the distance covariance between the treatment variable and
 #' the covariates and the energy distances between the treatment and covariates
 #' in the weighted sample and their distributions in the original sample. This
@@ -40,8 +40,7 @@
 #'
 #' For longitudinal treatments, the weights are the product of the weights
 #' estimated at each time point. This method is not guaranteed to yield optimal
-#' balance at each time point. NOTE: the use of energy balancing with
-#' longitudinal treatments has not been validated!
+#' balance at each time point. **NOTE: the use of energy balancing with longitudinal treatments has not been validated!**
 #'
 #' ## Sampling Weights
 #'
@@ -63,26 +62,27 @@
 #'
 #' M-estimation is not supported.
 #'
-#' @section Additional Arguments: The following following additional arguments
-#'   can be specified:
+#' @section Additional Arguments:
+#'
+#' The following following additional arguments can be specified:
+#'
 #' \describe{
-#'   \item{`dist.mat`}{the name of the method used to compute the distance matrix of the covariates or the numeric distance matrix itself. Allowable options include `"scaled_euclidean"` for the Euclidean (L2) distance on the scaled covariates (the default), `"mahalanobis"` for the Mahalanobis distance, and `"euclidean"` for the raw Euclidean distance. Abbreviations allowed. Note that some user-supplied distance matrices can cause the R session to abort due to a bug within \pkg{osqp}, so this argument should be used with caution. A distance matrix must be a square, symmetric, numeric matrix with zeros along the diagonal and a row and column for each unit. Can also be supplied as the output of a call to [dist()].
-#'   }
-#'   \item{`lambda`}{a positive numeric scalar used to penalize the square of the weights. This value divided by the square of the total sample size is added to the diagonal of the quadratic part of the loss function. Higher values favor weights with less variability. Note this is distinct from the lambda value described in Huling and Mak (2024), which penalizes the complexity of individual treatment rules rather than the weights, but does correspond to lambda from Huling et al. (2023). Default is .0001, which is essentially 0.
-#'   }
+#'   \item{`dist.mat`}{the name of the method used to compute the distance matrix of the covariates or the numeric distance matrix itself. Allowable options include `"scaled_euclidean"` for the Euclidean (L2) distance on the scaled covariates (the default), `"mahalanobis"` for the Mahalanobis distance, and `"euclidean"` for the raw Euclidean distance. Abbreviations allowed. Note that some user-supplied distance matrices can cause the R session to abort due to a bug within \pkg{osqp}, so this argument should be used with caution. A distance matrix must be a square, symmetric, numeric matrix with zeros along the diagonal and a row and column for each unit. Can also be supplied as the output of a call to [dist()].}
+#'   \item{`lambda`}{a positive numeric scalar used to penalize the square of the weights. This value divided by the square of the total sample size is added to the diagonal of the quadratic part of the loss function. Higher values favor weights with less variability. Note this is distinct from the lambda value described in Huling and Mak (2024), which penalizes the complexity of individual treatment rules rather than the weights, but does correspond to lambda from Huling et al. (2023). Default is .0001, which is essentially 0.}
+#'   \item{`moments`}{`integer`; the highest power of each covariate to be balanced. For example, if `moments = 3`, each covariate, its square, and its cube will be balanced. Can also be a named vector with a value for each covariate (e.g., `moments = c(x1 = 2, x2 = 4)`). Values greater than 1 for categorical covariates are ignored. Default is 0 to impose no constraint on balance.}
+#'   \item{`int`}{`logical`; whether first-order interactions of the covariates are to be balanced. Default is `FALSE`.}
+#'   \item{`tols`}{when `moments` is positive, a number corresponding to the maximum allowed standardized mean difference (for binary and multi-category treatments) or treatment-covariate correlation (for continuous treatments) allowed. Default is 0. Ignored when `moments = 0`.}
 #' }
-#'   For binary and multi-category treatments, the following additional
-#'   arguments can be specified:
+#'
+#' For binary and multi-category treatments, the following additional arguments can be specified:
 #'   \describe{
-#'     \item{`improved`}{`logical`; whether to use the improved energy balancing weights as described by Huling and Mak (2024) when `estimand = "ATE"`. This involves optimizing balance not only between each treatment group and the overall sample, but also between each pair of treatment groups. Huling and Mak (2024) found that the improved energy balancing weights generally outperformed standard energy balancing. Default is `TRUE`; set to `FALSE` to use the standard energy balancing weights instead (not recommended).
+#'     \item{`improved`}{`logical`; whether to use the improved energy balancing weights as described by Huling and Mak (2024) when `estimand = "ATE"`. This involves optimizing balance not only between each treatment group and the overall sample, but also between each pair of treatment groups. Huling and Mak (2024) found that the improved energy balancing weights generally outperformed standard energy balancing. Default is `TRUE`; set to `FALSE` to use the standard energy balancing weights instead (not recommended).}
+#'     \item{`quantile`}{a named list of quantiles (values between 0 and 1) for each continuous covariate, which are used to create additional variables that when balanced ensure balance on the corresponding quantile of the variable. For example, setting `quantile = list(x1 = c(.25, .5. , .75))` ensures the 25th, 50th, and 75th percentiles of `x1` in each treatment group will be balanced in the weighted sample. Can also be a single number (e.g., `.5`) or a vector (e.g., `c(.25, .5, .75)`) to request the same quantile(s) for all continuous covariates.
 #'     }
-#'   \item{`quantile`}{
-#'     A named list of quantiles (values between 0 and 1) for each continuous covariate, which are used to create additional variables that when balanced ensure balance on the corresponding quantile of the variable. For example, setting `quantile = list(x1 = c(.25, .5. , .75))` ensures the 25th, 50th, and 75th percentiles of `x1` in each treatment group will be balanced in the weighted sample. Can also be a single number (e.g., `.5`) or an unnamed list of length 1 (e.g., `list(c(.25, .5, .75))`) to request the same quantile(s) for all continuous covariates, or a named vector (e.g., `c(x1 = .5, x2 = .75)` to request one quantile for each covariate.
-#'   }
 #'   }
 #'
-#'   For continuous treatments, the following additional arguments can be
-#'   specified:
+#' For continuous treatments, the following additional arguments can be specified:
+#'
 #'   \describe{
 #'     \item{`d.moments`}{
 #'       The number of moments of the treatment and covariate distributions that are constrained to be the same in the weighted sample as in the original sample. For example, setting `d.moments = 3` ensures that the mean, variance, and skew of the treatment and covariates are the same in the weighted sample as in the unweighted sample. `d.moments` should be greater than or equal to `moments` and will be automatically set accordingly if not (or if not specified).
@@ -92,29 +92,29 @@
 #'     }
 #'   }
 #'
-#'   The `moments` argument functions differently for `method = "energy"` from
-#'   how it does with other methods. When unspecified or set to zero, energy
-#'   balancing weights are estimated as described by Huling and Mak (2024) for
-#'   binary and multi-category treatments or by Huling et al. (2023) for
-#'   continuous treatments. When `moments` is set to an integer larger than 0,
-#'   additional balance constraints on the requested moments of the covariates
-#'   are also included, guaranteeing exact moment balance on these covariates
-#'   while minimizing the energy distance of the weighted sample. For binary and
-#'   multi-category treatments, this involves exact balance on the means of the
-#'   entered covariates; for continuous treatments, this involves exact balance
-#'   on the treatment-covariate correlations of the entered covariates.
+#' The `moments` argument functions differently for `method = "energy"` from
+#' how it does with other methods. When unspecified or set to zero, energy
+#' balancing weights are estimated as described by Huling and Mak (2024) for
+#' binary and multi-category treatments or by Huling et al. (2023) for
+#' continuous treatments. When `moments` is set to an integer larger than 0,
+#' additional balance constraints on the requested moments of the covariates
+#' are also included, guaranteeing exact moment balance on these covariates
+#' while minimizing the energy distance of the weighted sample. For binary and
+#' multi-category treatments, this involves exact balance on the means of the
+#' entered covariates; for continuous treatments, this involves exact balance
+#' on the treatment-covariate correlations of the entered covariates. The constraint on exact balance can be relaxed using the `tols` argument.
 #'
-#'   Any other arguments will be passed to \pkgfun{osqp}{osqpSettings}. Some
-#'   defaults differ from those in `osqpSettings()`; see *Reproducibility*
-#'   below.
+#' Any other arguments will be passed to \pkgfun{osqp}{osqpSettings}. Some defaults differ from those in `osqpSettings()`; see *Reproducibility* section.
 #'
 #' @section Additional Outputs:
+#'
 #' \describe{
 #'   \item{`obj`}{When `include.obj = TRUE`, the output of the call to \pkgfun{osqp}{solve_osqp}, which contains the dual variables and convergence information.
 #'   }
 #' }
 #'
-#' @details Energy balancing is a method of estimating weights using
+#' @details
+#' Energy balancing is a method of estimating weights using
 #' optimization without a propensity score. The weights are the solution to a
 #' constrain quadratic optimization problem where the objective function
 #' concerns covariate balance as measured by the energy distance and (for
@@ -153,7 +153,8 @@
 #' reproducibility by default, `adaptive_rho_interval` is set to 10. See
 #' \pkgfun{osqp}{osqpSettings} for details.
 #'
-#' @note Sometimes the optimization can fail to converge because the problem is
+#' @note
+#' Sometimes the optimization can fail to converge because the problem is
 #' not convex. A warning will be displayed if so. In these cases, try simply
 #' re-fitting the weights without changing anything (but see the
 #' *Reproducibility* section above). If the method repeatedly fails, you should
@@ -168,52 +169,58 @@
 #' allows you to monitor the process and examine if the optimization is
 #' approaching convergence.
 #'
-#' @author Noah Greifer, using code from Jared Huling's
-#' \CRANpkg{independenceWeights} package for continuous treatments.
+#' If `min.w` is positive and you still get a warning about the presence of negative weights, try setting `eps` to a smaller number (e.g., to `1e-12`).
+#'
+#' As of version 1.5.0, `polish` is now set to `TRUE` by default. This should yield slightly improved solutions but may be a little slower.
+#'
+#' @author Noah Greifer, using code from Jared Huling's \CRANpkg{independenceWeights} package for continuous treatments.
 #'
 #' @seealso [weightit()], [weightitMSM()]
 #'
-#' @references ## Binary and multi-category treatments
+#' @references
+#' ## Binary and multi-category treatments
 #'
-#' Huling, J. D., & Mak, S. (2024). Energy balancing of covariate distributions.
-#' *Journal of Causal Inference*, 12(1). \doi{10.1515/jci-2022-0029}
+#' Huling, J. D., & Mak, S. (2024). Energy balancing of covariate distributions. *Journal of Causal Inference*, 12(1). \doi{10.1515/jci-2022-0029}
 #'
 #' ## Continuous treatments
 #'
-#' Huling, J. D., Greifer, N., & Chen, G. (2023). Independence weights for
-#' causal inference with continuous treatments. *Journal of the American
-#' Statistical Association*, 0(ja), 1–25. \doi{10.1080/01621459.2023.2213485}
+#' Huling, J. D., Greifer, N., & Chen, G. (2023). Independence weights for causal inference with continuous treatments. *Journal of the American Statistical Association*, 0(ja), 1–25. \doi{10.1080/01621459.2023.2213485}
 #'
-#' @examplesIf requireNamespace("osqp", quietly = TRUE)
-#' library("cobalt")
+#' @examplesIf rlang::is_installed("osqp")
 #' data("lalonde", package = "cobalt")
 #'
 #' #Balancing covariates between treatment groups (binary)
 #' (W1 <- weightit(treat ~ age + educ + married +
 #'                   nodegree + re74, data = lalonde,
 #'                 method = "energy", estimand = "ATE"))
+#'
 #' summary(W1)
-#' bal.tab(W1)
+#'
+#' cobalt::bal.tab(W1)
 #'
 #' #Balancing covariates with respect to race (multi-category)
 #' (W2 <- weightit(race ~ age + educ + married +
 #'                   nodegree + re74, data = lalonde,
 #'                 method = "energy", estimand = "ATT",
 #'                 focal = "black"))
+#'
 #' summary(W2)
-#' bal.tab(W2)
-#' \donttest{
-#'   #Balancing covariates with respect to re75 (continuous)
-#'   (W3 <- weightit(re75 ~ age + educ + married +
+#'
+#' cobalt::bal.tab(W2)
+#'
+#' \donttest{#Balancing covariates with respect to re75 (continuous)
+#' (W3 <- weightit(re75 ~ age + educ + married +
 #'                     nodegree + re74, data = lalonde,
-#'                   method = "energy", moments = 1))
-#'   summary(W3)
-#'   bal.tab(W3, poly = 2)
-#' }
+#'                   method = "energy", moments = 1,
+#'                   tols = .01))
+#'
+#' summary(W3)
+#'
+#' cobalt::bal.tab(W3, poly = 2)}
 NULL
 
 weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
-                            missing, moments, int, verbose, ...) {
+                            missing, verbose, ...) {
 
   missing <- .process_missing2(missing, covs)
 
@@ -224,12 +231,15 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
   d <- ...get("dist.mat", "scaled_euclidean")
 
   if (chk::vld_string(d)) {
-    dist.covs <- transform_covariates(data = covs, method = d,
-                                      s.weights = s.weights, discarded = !subset)
-    d <- unname(eucdist_internal(dist.covs))
+    d <- transform_covariates(data = covs, method = d,
+                              s.weights = s.weights, discarded = !subset) |>
+      eucdist_internal() |>
+      unname()
   }
   else {
-    if (inherits(d, "dist")) d <- as.matrix(d)
+    if (inherits(d, "dist")) {
+      d <- as.matrix(d)
+    }
 
     if (!is.matrix(d) || !all(dim(d) == length(treat)) ||
         !all(check_if_zero(diag(d))) ||
@@ -240,7 +250,7 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
     }
   }
 
-  d <- unname(d[subset, subset])
+  d <- unname(d[subset, subset, drop = FALSE])
 
   covs <- covs[subset, , drop = FALSE]
   treat <- treat[subset]
@@ -260,6 +270,17 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
   lambda <- ...get("lambda", 1e-4)
   chk::chk_number(lambda)
 
+  moments <- ...get("moments", 0)
+  int <- isTRUE(...get("int", FALSE))
+  quantile <- ...get("quantile")
+  add_constraints <- any(moments > 0) || int || is_not_null(quantile)
+
+  if (add_constraints) {
+    tols <- ...get("tols", 0)
+    chk::chk_number(tols)
+    tols <- abs(tols)
+  }
+
   t0 <- which(treat == 0)
   t1 <- which(treat == 1)
 
@@ -292,80 +313,131 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
     lvec <- c(rep.int(min.w, n), 1, 1)
     uvec <- c(ifelse(check_if_zero(s.weights), min.w, Inf), 1, 1)
 
-    if (moments > 0 || int || is_not_null(...get("quantile"))) {
+    if (add_constraints) {
       #Exactly balance moments, interactions, and/or quantiles
-      covs <- cbind(.int_poly_f(covs, poly = moments, int = int, center = TRUE),
-                    .quantile_f(covs, qu = ...get("quantile"), s.weights = s.weights))
+      covs <- .apply_moments_int_quantile(covs,
+                                          moments = moments,
+                                          int = int,
+                                          quantile = quantile,
+                                          s.weights = s.weights)
 
       targets <- col.w.m(covs, s.weights)
 
+      sds <- rep.int(1, ncol(covs))
+
+      if (tols > 0) {
+        bin.vars <- is_binary_col(covs)
+
+        if (!all(bin.vars)) {
+          sds[!bin.vars] <- sqrt(colMeans(rbind(col.w.v(covs[t1, !bin.vars, drop = FALSE], w = s.weights[t1]),
+                                                col.w.v(covs[t0, !bin.vars, drop = FALSE], w = s.weights[t0]))))
+        }
+      }
+
       Amat <- cbind(Amat, covs * s.weights_n_0, covs * s.weights_n_1)
-      lvec <- c(lvec, targets, targets)
-      uvec <- c(uvec, targets, targets)
+      lvec <- c(lvec, targets - sds * tols / 2, targets - sds * tols / 2)
+      uvec <- c(uvec, targets + sds * tols / 2, targets + sds * tols / 2)
     }
   }
   else if (estimand == "ATT") {
     nn <- tcrossprod(s.weights_n_0[t0])
 
-    P <- -d[t0, t0] * nn
+    P <- -d[t0, t0, drop = FALSE] * nn
 
-    q <- 2 * (s.weights_n_1[t1] %*% d[t1, t0]) * s.weights_n_0[t0]
+    q <- 2 * (s.weights_n_1[t1] %*% d[t1, t0, drop = FALSE]) * s.weights_n_0[t0]
 
     Amat <- cbind(diag(n0), s.weights_n_0[t0])
     lvec <- c(rep.int(min.w, n0), 1)
     uvec <- c(ifelse(check_if_zero(s.weights[t0]), min.w, Inf), 1)
 
-    if (moments > 0 || int || is_not_null(...get("quantile"))) {
+    if (add_constraints) {
       #Exactly balance moments, interactions, and/or quantiles
-      covs <- cbind(.int_poly_f(covs, poly = moments, int = int, center = TRUE),
-                    .quantile_f(covs, qu = ...get("quantile"), s.weights = s.weights,
-                                focal = 1, treat = treat))
+      covs <- .apply_moments_int_quantile(covs,
+                                          moments = moments,
+                                          int = int,
+                                          quantile = quantile,
+                                          s.weights = s.weights, focal = 1,
+                                          treat = treat)
 
       targets <- col.w.m(covs[t1, , drop = FALSE], s.weights[t1])
 
-      Amat <- cbind(Amat, covs[t0, , drop = FALSE] * s.weights_n_0[t0])
+      sds <- rep.int(1, ncol(covs))
 
-      lvec <- c(lvec, targets)
-      uvec <- c(uvec, targets)
+      if (tols > 0) {
+        bin.vars <- is_binary_col(covs)
+
+        if (!all(bin.vars)) {
+          sds[!bin.vars] <- sqrt(col.w.v(covs[t1, !bin.vars, drop = FALSE], w = s.weights[t1]))
+        }
+      }
+
+      Amat <- cbind(Amat, covs[t0, , drop = FALSE] * s.weights_n_0[t0])
+      lvec <- c(lvec, targets - sds * tols)
+      uvec <- c(uvec, targets + sds * tols)
     }
   }
   else if (estimand == "ATC") {
     nn <- tcrossprod(s.weights_n_1[t1])
 
-    P <- -d[t1, t1] * nn
+    P <- -d[t1, t1, drop = FALSE] * nn
 
-    q <- 2 * (s.weights_n_0[t0] %*% d[t0, t1]) * s.weights_n_1[t1]
+    q <- 2 * (s.weights_n_0[t0] %*% d[t0, t1, drop = FALSE]) * s.weights_n_1[t1]
 
     Amat <- cbind(diag(n1), s.weights_n_1[t1])
     lvec <- c(rep.int(min.w, n1), 1)
     uvec <- c(ifelse(check_if_zero(s.weights[t1]), min.w, Inf), 1)
 
-    if (moments > 0 || int || is_not_null(...get("quantile"))) {
+    if (add_constraints) {
       #Exactly balance moments, interactions, and/or quantiles
-      covs <- cbind(.int_poly_f(covs, poly = moments, int = int, center = TRUE),
-                    .quantile_f(covs, qu = ...get("quantile"), s.weights = s.weights,
-                                focal = 0, treat = treat))
+      covs <- .apply_moments_int_quantile(covs,
+                                          moments = moments,
+                                          int = int,
+                                          quantile = quantile,
+                                          s.weights = s.weights, focal = 0,
+                                          treat = treat)
 
       targets <- col.w.m(covs[t0, , drop = FALSE], s.weights[t0])
 
-      Amat <- cbind(Amat, covs[t1, , drop = FALSE] * s.weights_n_1[t1])
+      if (tols > 0) {
+        bin.vars <- is_binary_col(covs)
 
-      lvec <- c(lvec, targets)
-      uvec <- c(uvec, targets)
+        if (!all(bin.vars)) {
+          sds[!bin.vars] <- sqrt(col.w.v(covs[t0, !bin.vars, drop = FALSE], w = s.weights[t0]))
+        }
+      }
+
+      Amat <- cbind(Amat, covs[t1, , drop = FALSE] * s.weights_n_1[t1])
+      lvec <- c(lvec, targets - sds * tols)
+      uvec <- c(uvec, targets + sds * tols)
     }
   }
 
   #Add weight penalty
   if (lambda < 0) {
+    # es <- eigen(P, symmetric = TRUE)
+    # esv <- es$values
+    #
+    # tol <- nrow(P) * max(abs(esv)) * .Machine$double.eps
+    #
+    # if (!all(esv > tol)) {
+    #   tau <- pmax(0, 2 * tol - esv)
+    #   P <- P + tcrossprod(es$vectors %*% diag(sqrt(tau), nrow(P)))
+    # }
+
     #Find lambda to make P PSD
     e <- eigen(P, symmetric = TRUE, only.values = TRUE)
     e.min <- min(e$values)
+
     if (e.min < 0) {
-      lambda <- -e.min * n^2
+      diag(P) <- diag(P) - e.min + .Machine$double.eps
     }
   }
-
-  diag(P) <- diag(P) + lambda / n^2
+  else if (lambda != 0) {
+    diag(P) <- switch(estimand,
+                      ATE = diag(P) + lambda * (s.weights_n_0 + s.weights_n_1)^2 / 2,
+                      ATT = diag(P) + lambda * s.weights_n_0[t0]^2 / 2,
+                      ATC = diag(P) + lambda * s.weights_n_1[t1]^2 / 2)
+  }
 
   A <- ...mget(names(formals(osqp::osqpSettings)))
 
@@ -375,7 +447,7 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
     if (is_null(A[["eps_rel"]])) A[["eps_rel"]] <- ...get("eps")
   }
 
-  if (is_null(A[["max_iter"]])) A[["max_iter"]] <- 4e3L
+  if (is_null(A[["max_iter"]])) A[["max_iter"]] <- 5e4L
   chk::chk_count(A[["max_iter"]], "`max_iter`")
   chk::chk_lt(A[["max_iter"]], Inf, "`max_iter`")
   if (is_null(A[["eps_abs"]])) A[["eps_abs"]] <- 1e-8
@@ -386,6 +458,8 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
   chk::chk_number(A[["time_limit"]], "`time_limit`")
   if (is_null(A[["adaptive_rho_interval"]])) A[["adaptive_rho_interval"]] <- 10L
   chk::chk_count(A[["adaptive_rho_interval"]], "`adaptive_rho_interval`")
+  if (is_null(A[["polish"]])) A[["polish"]] <- TRUE
+  chk::chk_flag(A[["polish"]], "`polish`")
   A[["verbose"]] <- TRUE
 
   options.list <- do.call(osqp::osqpSettings, A)
@@ -419,7 +493,10 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
     w <- opt.out$x
   }
 
-  w[w <= min.w] <- min.w
+  # Shrink tiny weights to 0
+  if (abs(min.w) < .Machine$double.eps) {
+    w[abs(w) < .Machine$double.eps] <- 0
+  }
 
   opt.out$lambda <- lambda
 
@@ -427,7 +504,7 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
 }
 
 weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, focal,
-                                  missing, moments, int, verbose, ...) {
+                                  missing, verbose, ...) {
 
   missing <- .process_missing2(missing, covs)
 
@@ -438,12 +515,15 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
   d <- ...get("dist.mat", "scaled_euclidean")
 
   if (chk::vld_string(d)) {
-    dist.covs <- transform_covariates(data = covs, method = d,
-                                      s.weights = s.weights, discarded = !subset)
-    d <- unname(eucdist_internal(dist.covs))
+    d <- transform_covariates(data = covs, method = d,
+                              s.weights = s.weights, discarded = !subset) |>
+      eucdist_internal() |>
+      unname()
   }
   else {
-    if (inherits(d, "dist")) d <- as.matrix(d)
+    if (inherits(d, "dist")) {
+      d <- as.matrix(d)
+    }
 
     if (!is.matrix(d) || !all(dim(d) == length(treat)) ||
         !all(check_if_zero(diag(d))) || any(d < 0) ||
@@ -454,7 +534,7 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
 
   }
 
-  d <- unname(d[subset, subset])
+  d <- unname(d[subset, subset, drop = FALSE])
 
   covs <- covs[subset, , drop = FALSE]
   treat <- factor(treat[subset])
@@ -472,11 +552,26 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
   lambda <- ...get("lambda", 1e-4)
   chk::chk_number(lambda)
 
+  moments <- ...get("moments", 0)
+  int <- isTRUE(...get("int", FALSE))
+  quantile <- ...get("quantile")
+  add_constraints <- any(moments > 0) || int || is_not_null(quantile)
+
+  if (add_constraints) {
+    tols <- ...get("tols", 0)
+    chk::chk_number(tols)
+    tols <- abs(tols)
+  }
+
+  treat_t <- matrix(0, nrow = n, ncol = length(levels_treat),
+                    dimnames = list(NULL, levels_treat))
+
   for (t in levels_treat) {
-    s.weights[treat == t] <- s.weights[treat == t] / mean_fast(s.weights[treat == t])
+    in_t <- which(treat == t)
+    s.weights[in_t] <- s.weights[in_t] / mean_fast(s.weights[in_t])
+    treat_t[in_t, t] <- 1
   }
 
-  treat_t <- vapply(levels_treat, function(t) treat == t, logical(n))
   n_t <- colSums(treat_t)
 
   s.weights_n_t <- vapply(levels_treat, function(t) treat_t[, t] * s.weights / n_t[t],
@@ -504,16 +599,32 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
     lvec <- c(rep.int(min.w, n), rep.int(1, length(levels_treat)))
     uvec <- c(ifelse(check_if_zero(s.weights), min.w, Inf), rep.int(1, length(levels_treat)))
 
-    if (moments > 0 || int || is_not_null(...get("quantile"))) {
+    if (add_constraints) {
       #Exactly balance moments, interactions, and/or quantiles
-      covs <- cbind(.int_poly_f(covs, poly = moments, int = int, center = TRUE),
-                    .quantile_f(covs, qu = ...get("quantile"), s.weights = s.weights))
+      covs <- .apply_moments_int_quantile(covs,
+                                          moments = moments,
+                                          int = int,
+                                          quantile = quantile,
+                                          s.weights = s.weights)
 
       targets <- col.w.m(covs, s.weights)
 
+      sds <- rep.int(1, ncol(covs))
+
+      if (tols > 0) {
+        bin.vars <- is_binary_col(covs)
+
+        if (!all(bin.vars)) {
+          sds[!bin.vars] <- sqrt(colMeans(do.call("rbind",
+                                                  lapply(levels_treat, function(t) {
+                                                    col.w.v(covs[treat == t, !bin.vars, drop = FALSE], w = s.weights[treat == t])
+                                                  }))))
+        }
+      }
+
       Amat <- cbind(Amat, do.call("cbind", apply(s.weights_n_t, 2L, function(x) covs * x, simplify = FALSE)))
-      lvec <- c(lvec, rep.int(targets, length(levels_treat)))
-      uvec <- c(uvec, rep.int(targets, length(levels_treat)))
+      lvec <- c(lvec, rep.int(targets - sds * tols / 2, length(levels_treat)))
+      uvec <- c(uvec, rep.int(targets + sds * tols / 2, length(levels_treat)))
     }
   }
   else {
@@ -522,28 +633,41 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
 
     nn <- tcrossprod(s.weights_n_t[!in_focal, non_focal, drop = FALSE])
 
-    P <- -d[!in_focal, !in_focal] * nn
+    P <- -d[!in_focal, !in_focal, drop = FALSE] * nn
 
-    q <- 2 * (s.weights_n_t[in_focal, focal] %*% d[in_focal, !in_focal]) *
+    q <- 2 * (s.weights_n_t[in_focal, focal] %*% d[in_focal, !in_focal, drop = FALSE]) *
       rowSums(s.weights_n_t[!in_focal, non_focal, drop = FALSE])
 
     Amat <- cbind(diag(sum(!in_focal)), s.weights_n_t[!in_focal, non_focal])
     lvec <- c(rep.int(min.w, sum(!in_focal)), rep.int(1, length(non_focal)))
     uvec <- c(ifelse(check_if_zero(s.weights[!in_focal]), min.w, Inf), rep.int(1, length(non_focal)))
 
-    if (moments > 0 || int || is_not_null(...get("quantile"))) {
+    if (add_constraints) {
       #Exactly balance moments, interactions, and/or quantiles
-      covs <- cbind(.int_poly_f(covs, poly = moments, int = int, center = TRUE),
-                    .quantile_f(covs, qu = ...get("quantile"), s.weights = s.weights,
-                                focal = focal, treat = treat))
+      covs <- .apply_moments_int_quantile(covs,
+                                          moments = moments,
+                                          int = int,
+                                          quantile = quantile,
+                                          s.weights = s.weights, focal = focal,
+                                          treat = treat)
 
       targets <- col.w.m(covs[in_focal, , drop = FALSE], s.weights[in_focal])
 
+      sds <- rep.int(1, ncol(covs))
+
+      if (tols > 0) {
+        bin.vars <- is_binary_col(covs)
+
+        if (!all(bin.vars)) {
+          sds[!bin.vars] <- sqrt(col.w.v(covs[in_focal, !bin.vars, drop = FALSE], w = s.weights[in_focal]))
+        }
+      }
+
       Amat <- cbind(Amat, do.call("cbind", apply(s.weights_n_t[!in_focal, non_focal, drop = FALSE], 2L,
                                                  function(x) covs[!in_focal, , drop = FALSE] * x,
                                                  simplify = FALSE)))
-      lvec <- c(lvec, rep.int(targets, length(non_focal)))
-      uvec <- c(uvec, rep.int(targets, length(non_focal)))
+      lvec <- c(lvec, rep.int(targets - sds * tols, length(non_focal)))
+      uvec <- c(uvec, rep.int(targets + sds * tols, length(non_focal)))
     }
   }
 
@@ -567,7 +691,7 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
     if (is_null(A[["eps_rel"]])) A[["eps_rel"]] <- ...get("eps")
   }
 
-  if (is_null(A[["max_iter"]])) A[["max_iter"]] <- 4e3L
+  if (is_null(A[["max_iter"]])) A[["max_iter"]] <- 5e4L
   chk::chk_count(A[["max_iter"]], "`max_iter`")
   chk::chk_lt(A[["max_iter"]], Inf, "`max_iter`")
   if (is_null(A[["eps_abs"]])) A[["eps_abs"]] <- 1e-8
@@ -578,6 +702,8 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
   chk::chk_number(A[["time_limit"]], "`time_limit`")
   if (is_null(A[["adaptive_rho_interval"]])) A[["adaptive_rho_interval"]] <- 10L
   chk::chk_count(A[["adaptive_rho_interval"]], "`adaptive_rho_interval`")
+  if (is_null(A[["polish"]])) A[["polish"]] <- TRUE
+  chk::chk_flag(A[["polish"]], "`polish`")
   A[["verbose"]] <- TRUE
 
   options.list <- do.call(osqp::osqpSettings, A)
@@ -607,14 +733,17 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
     w[treat != focal] <- opt.out$x
   }
 
-  w[w <= min.w] <- min.w
+  # Shrink tiny weights to 0
+  if (abs(min.w) < .Machine$double.eps) {
+    w[abs(w) < .Machine$double.eps] <- 0
+  }
 
   opt.out$lambda <- lambda
 
   list(w = w, fit.obj = opt.out)
 }
 
-weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, moments, int, verbose, ...) {
+weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, verbose, ...) {
 
   missing <- .process_missing2(missing, covs)
 
@@ -625,12 +754,15 @@ weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, moment
   Xdist <- ...get("dist.mat", "scaled_euclidean")
 
   if (chk::vld_string(Xdist)) {
-    dist.covs <- transform_covariates(data = covs, method = Xdist,
-                                      s.weights = s.weights, discarded = !subset)
-    Xdist <- unname(eucdist_internal(X = dist.covs))
+    Xdist <- transform_covariates(data = covs, method = Xdist,
+                                  s.weights = s.weights, discarded = !subset) |>
+      eucdist_internal() |>
+      unname()
   }
   else {
-    if (inherits(Xdist, "dist")) Xdist <- as.matrix(Xdist)
+    if (inherits(Xdist, "dist")) {
+      Xdist <- as.matrix(Xdist)
+    }
 
     if (!is.matrix(Xdist) || !all(dim(Xdist) == length(treat)) ||
         !all(check_if_zero(diag(Xdist))) || any(Xdist < 0) ||
@@ -640,7 +772,7 @@ weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, moment
     }
   }
 
-  Xdist <- unname(Xdist[subset, subset])
+  Xdist <- unname(Xdist[subset, subset, drop = FALSE])
 
   if (is_null(...get("treat.dist.mat"))) {
     Adist <- eucdist_internal(X = treat / sqrt(col.w.v(treat, s.weights)))
@@ -648,7 +780,9 @@ weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, moment
   else {
     Adist <- ...get("treat.dist.mat")
 
-    if (inherits(Adist, "dist")) Adist <- as.matrix(Adist)
+    if (inherits(Adist, "dist")) {
+      Adist <- as.matrix(Adist)
+    }
 
     if (!is.matrix(Adist) || !all(dim(Adist) == length(treat)) ||
         !all(check_if_zero(diag(Adist))) || any(Adist < 0) ||
@@ -657,7 +791,7 @@ weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, moment
     }
   }
 
-  Adist <- unname(Adist[subset, subset])
+  Adist <- unname(Adist[subset, subset, drop = FALSE])
 
   covs <- covs[subset, , drop = FALSE]
   treat <- treat[subset]
@@ -675,6 +809,16 @@ weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, moment
   lambda <- ...get("lambda", 1e-4)
   chk::chk_number(lambda)
 
+  moments <- ...get("moments", 0)
+  int <- isTRUE(...get("int", FALSE))
+  add_constraints <- any(moments > 0) || int
+
+  if (add_constraints) {
+    tols <- ...get("tols", 0)
+    chk::chk_number(tols)
+    tols <- abs(tols)
+  }
+
   d.moments <- max(...get("d.moments", 0), moments)
   chk::chk_count(d.moments)
 
@@ -703,47 +847,46 @@ weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, moment
 
   Q_energy_X_adj <- 1 - Q_energy_A_adj
 
-  PebA <- PebA * Q_energy_A_adj
-  PebX <- PebX * Q_energy_X_adj
+  PebA[] <- PebA * Q_energy_A_adj
+  PebX[] <- PebX * Q_energy_X_adj
 
-  qebA <- qebA * Q_energy_A_adj
-  qebX <- qebX * Q_energy_X_adj
+  qebA[] <- qebA * Q_energy_A_adj
+  qebX[] <- qebX * Q_energy_X_adj
 
   P <- Pdcow + PebA + PebX
   q <- qebA + qebX
 
-  P <- P * tcrossprod(s.weights)
-  q <- q * s.weights
+  P[] <- P * tcrossprod(s.weights)
+  q[] <- q * s.weights
 
   Amat <- cbind(diag(n), s.weights)
   lvec <- c(rep.int(min.w, n), n)
   uvec <- c(ifelse(sw0, min.w, Inf), n)
 
   if (d.moments > 0) {
-    d.covs <- .int_poly_f(covs, poly = d.moments)
+    d.covs <- .apply_moments_int_quantile(covs, moments = d.moments)
     d.treat <- cbind(poly(treat, degree = d.moments))
 
-    d.covs <- center(d.covs, col.w.m(d.covs, s.weights))
-    d.treat <- center(d.treat, col.w.m(d.treat, s.weights))
+    d.covs <- scale_w(d.covs, s.weights)
+    d.treat <- scale_w(d.treat, s.weights)
 
-    Amat <- cbind(Amat, d.covs * s.weights, d.treat * s.weights)
+    Amat <- cbind(Amat, d.covs * s.weights / n, d.treat * s.weights / n)
     lvec <- c(lvec, rep.int(0, ncol(d.covs)), rep.int(0, ncol(d.treat)))
     uvec <- c(uvec, rep.int(0, ncol(d.covs)), rep.int(0, ncol(d.treat)))
   }
 
-  if (moments > 0 || int) {
-    covs <- .int_poly_f(covs, poly = moments, int = int)
+  if (add_constraints) {
+    covs <- .apply_moments_int_quantile(covs,
+                                        moments = moments,
+                                        int = int)
 
-    X.means <- col.w.m(covs, s.weights)
-    A.mean <- w.m(treat, s.weights)
+    covs <- scale_w(covs, s.weights)
+    treat <- scale_w(treat, s.weights)
 
-    covs <- center(covs, X.means)
-    treat <- treat - A.mean
+    Amat <- cbind(Amat, covs * treat * s.weights / n)
 
-    Amat <- cbind(Amat, covs * treat * s.weights)
-
-    lvec <- c(lvec, rep.int(0, ncol(covs)))
-    uvec <- c(uvec, rep.int(0, ncol(covs)))
+    lvec <- c(lvec, rep.int(-tols, ncol(covs)))
+    uvec <- c(uvec, rep.int(tols, ncol(covs)))
   }
 
   #Add weight penalty
@@ -752,30 +895,32 @@ weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, moment
     e <- eigen(P, symmetric = TRUE, only.values = TRUE)
     e.min <- min(e$values)
 
-    lambda <- -e.min * n^2
+    if (e.min < 0) {
+      lambda <- -e.min * n^2
+    }
   }
 
   diag(P) <- diag(P) + lambda / n^2
 
   A <- ...mget(names(formals(osqp::osqpSettings)))
 
-  if (is_not_null(...get("eps"))) {
-    chk::chk_number(...get("eps"), "`eps`")
-    if (is_null(A[["eps_abs"]])) A[["eps_abs"]] <- ...get("eps")
-    if (is_null(A[["eps_rel"]])) A[["eps_rel"]] <- ...get("eps")
-  }
+  eps <- ...get("eps", 1e-8)
+
+  chk::chk_number(eps)
+  if (is_null(A[["eps_abs"]])) A[["eps_abs"]] <- eps
+  if (is_null(A[["eps_rel"]])) A[["eps_rel"]] <- eps
 
   if (is_null(A[["max_iter"]])) A[["max_iter"]] <- 5e4L
   chk::chk_count(A[["max_iter"]], "`max_iter`")
   chk::chk_lt(A[["max_iter"]], Inf, "`max_iter`")
-  if (is_null(A[["eps_abs"]])) A[["eps_abs"]] <- 1e-8
   chk::chk_number(A[["eps_abs"]], "`eps_abs`")
-  if (is_null(A[["eps_rel"]])) A[["eps_rel"]] <- 1e-6
   chk::chk_number(A[["eps_rel"]], "`eps_rel`")
   if (is_null(A[["time_limit"]])) A[["time_limit"]] <- 0
   chk::chk_number(A[["time_limit"]], "`time_limit`")
   if (is_null(A[["adaptive_rho_interval"]])) A[["adaptive_rho_interval"]] <- 10L
   chk::chk_count(A[["adaptive_rho_interval"]], "`adaptive_rho_interval`")
+  if (is_null(A[["polish"]])) A[["polish"]] <- TRUE
+  chk::chk_flag(A[["polish"]], "`polish`")
   A[["verbose"]] <- TRUE
 
   options.list <- do.call(osqp::osqpSettings, A)
@@ -798,7 +943,11 @@ weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, moment
   }
 
   w <- opt.out$x
-  w[w <= min.w] <- min.w
+
+  # Shrink tiny weights to 0
+  if (abs(min.w) < .Machine$double.eps) {
+    w[abs(w) < .Machine$double.eps] <- 0
+  }
 
   opt.out$lambda <- lambda
 
