@@ -1,9 +1,9 @@
 #' Propensity Score Weighting Using SuperLearner
 #' @name method_super
-#' @aliases method_super
 #' @usage NULL
 #'
-#' @description This page explains the details of estimating weights from
+#' @description
+#' This page explains the details of estimating weights from
 #' SuperLearner-based propensity scores by setting `method = "super"` in the
 #' call to [weightit()] or [weightitMSM()]. This method can be used with binary,
 #' multi-category, and continuous treatments.
@@ -42,8 +42,7 @@
 #' For continuous treatments, the generalized propensity score is estimated
 #' using \pkgfun{SuperLearner}{SuperLearner}. In addition, kernel density
 #' estimation can be used instead of assuming a normal density for the numerator
-#' and denominator of the generalized propensity score by setting `density =
-#' "kernel"`. Other arguments to [density()] can be specified to refine the
+#' and denominator of the generalized propensity score by setting `density = "kernel"`. Other arguments to [density()] can be specified to refine the
 #' density estimation parameters. `plot = TRUE` can be specified to plot the
 #' density for the numerator and denominator, which can be helpful in diagnosing
 #' extreme weights.
@@ -71,20 +70,31 @@
 #' M-estimation is not supported.
 #'
 #' @section Additional Arguments:
+#'
 #' \describe{
 #'   \item{`discrete`}{if `TRUE`, uses discrete SuperLearner, which simply selects the best performing method. Default `FALSE`, which finds the optimal combination of predictions for the libraries using `SL.method`.}
 #' }
 #'
-#'   An argument to `SL.library` **must** be supplied. To see a list of
-#'   available entries, use \pkgfun{SuperLearner}{listWrappers}.
+#' An argument to `SL.library` **must** be supplied. To see a list of
+#' available entries, use \pkgfun{SuperLearner}{listWrappers}.
 #'
-#'   All arguments to \pkgfun{SuperLearner}{SuperLearner} can be passed through
-#'   `weightit()` or `weightitMSM()`, with the following exceptions:
+#' All arguments to \pkgfun{SuperLearner}{SuperLearner} can be passed through
+#' `weightit()` or `weightitMSM()`, with the following exceptions:
 #'
 #'   * `obsWeights` is ignored because sampling weights are passed using `s.weights`.
 #'   * `method` in `SuperLearner()` is replaced with the argument `SL.method` in `weightit()`.
 #'
-#'   For continuous treatments only, the following arguments may be supplied:
+#' For binary and multi-category treatments, the following arguments may be supplied:
+#' \describe{
+#'   \item{`subclass`}{`integer`; the number of subclasses to use for computing
+#'   weights using marginal mean weighting through stratification (MMWS). If `NULL`,
+#'   standard inverse probability weights (and their extensions) will be
+#'   computed; if a number greater than 1, subclasses will be formed and weights
+#'   will be computed based on subclass membership. See [get_w_from_ps()] for
+#'   details and references.}
+#' }
+#'
+#' For continuous treatments, the following arguments may be supplied:
 #'   \describe{
 #'     \item{`density`}{A function corresponding to the conditional density of the treatment. The standardized residuals of the treatment model will be fed through this function to produce the numerator and denominator of the generalized propensity score weights. If blank, [dnorm()] is used as recommended by Robins et al. (2000). This can also be supplied as a string containing the name of the function to be called. If the string contains underscores, the call will be split by the underscores and the latter splits will be supplied as arguments to the second argument and beyond. For example, if `density = "dt_2"` is specified, the density used will be that of a t-distribution with 2 degrees of freedom. Using a t-distribution can be useful when extreme outcome values are observed (Naimi et al., 2014).
 #'
@@ -104,7 +114,7 @@
 #'   combination of the predictions optimize a balance criterion, which must be
 #'   set with the `criterion` argument, described below.
 #'   \describe{
-#'     \item{`criterion`}{A string describing the balance criterion used to select the best weights. See \pkgfun{cobalt}{bal.compute} for allowable options for each treatment type. For binary and multi-category treatments, the default is `"smd.mean"`, which minimizes the average absolute standard mean difference among the covariates between treatment groups. For continuous treatments, the default is `"p.mean"`, which minimizes the average absolute Pearson correlation between the treatment and covariates.
+#'     \item{`criterion`}{a string describing the balance criterion used to select the best weights. See \pkgfun{cobalt}{bal.compute} for allowable options for each treatment type. For binary and multi-category treatments, the default is `"smd.mean"`, which minimizes the average absolute standard mean difference among the covariates between treatment groups. For continuous treatments, the default is `"p.mean"`, which minimizes the average absolute Pearson correlation between the treatment and covariates.
 #'     }
 #'   }
 #'   Note that this implementation differs from that of Pirracchio and Carone
@@ -133,22 +143,22 @@
 #'   }
 #' }
 #'
-#' @details SuperLearner works by fitting several machine learning models to the
+#' @details
+#' SuperLearner works by fitting several machine learning models to the
 #' treatment and covariates and then taking a weighted combination of the
 #' generated predicted values to use as the propensity scores, which are then
 #' used to construct weights. The machine learning models used are supplied
 #' using the `SL.library` argument; the more models are supplied, the higher the
 #' chance of correctly modeling the propensity score. It is a good idea to
-#' include parameteric models, flexible and tree-based models, and regularized
+#' include parametric models, flexible and tree-based models, and regularized
 #' models among the models selected. The predicted values are combined using the
 #' method supplied in the `SL.method` argument (which is nonnegative least
 #' squares by default). A benefit of SuperLearner is that, asymptotically, it is
 #' guaranteed to perform as well as or better than the best-performing method
-#' included in the library. Using Balance SuperLearner by setting `SL.method =
-#' "method.balance"` works by selecting the combination of predicted values that
-#' minimizes an imbalance measure.
+#' included in the library. Using Balance SuperLearner by setting `SL.method = "method.balance"` works by selecting the combination of predicted values that minimizes an imbalance measure.
 #'
-#' @note Some methods formerly available in \pkg{SuperLearner} are now in
+#' @note
+#' Some methods formerly available in \pkg{SuperLearner} are now in
 #' \pkg{SuperLearnerExtra}, which can be found on GitHub at
 #' \url{https://github.com/ecpolley/SuperLearnerExtra}.
 #'
@@ -166,7 +176,8 @@
 #'
 #' @seealso [weightit()], [weightitMSM()], [get_w_from_ps()]
 #'
-#' @references ## Binary treatments
+#' @references
+#' ## Binary treatments
 #'
 #' Pirracchio, R., Petersen, M. L., & van der Laan, M. (2015). Improving
 #' Propensity Score Estimators’ Robustness to Model Misspecification Using Super
@@ -190,45 +201,50 @@
 #'
 #' See [`method_glm`] for additional references.
 #'
-#' @examplesIf all(sapply(c("SuperLearner", "MASS"), requireNamespace, quietly = TRUE))
-#' \donttest{library("cobalt")
-#' data("lalonde", package = "cobalt")
+#' @examplesIf rlang::is_installed(c("SuperLearner", "MASS"))
+#' \donttest{data("lalonde", package = "cobalt")
 #'
 #' #Note: for time, all exmaples use a small set of
 #' #      learners. Many more should be added if
 #' #      possible, including a variety of model
 #' #      types (e.g., parametric, flexible, tree-
-#' #.     based, regularized, etc.)
+#' #      based, regularized, etc.)
 #'
-#' #Balancing covariates between treatment groups (binary)
+#' # Balancing covariates between treatment groups (binary)
 #' (W1 <- weightit(treat ~ age + educ + married +
 #'                   nodegree + re74, data = lalonde,
 #'                 method = "super", estimand = "ATT",
 #'                 SL.library = c("SL.glm", "SL.stepAIC",
 #'                                "SL.glm.interaction")))
-#' summary(W1)
-#' bal.tab(W1)
 #'
-#' #Balancing covariates with respect to race (multi-category)
+#' summary(W1)
+#'
+#' cobalt::bal.tab(W1)
+#'
+#' # Balancing covariates with respect to race (multi-category)
 #' (W2 <- weightit(race ~ age + educ + married +
 #'                   nodegree + re74, data = lalonde,
 #'                 method = "super", estimand = "ATE",
 #'                 SL.library = c("SL.glm", "SL.stepAIC",
 #'                                "SL.glm.interaction")))
-#' summary(W2)
-#' bal.tab(W2)
 #'
-#' #Balancing covariates with respect to re75 (continuous)
-#' #assuming t(8) conditional density for treatment
+#' summary(W2)
+#'
+#' cobalt::bal.tab(W2)
+#'
+#' # Balancing covariates with respect to re75 (continuous)
+#' # assuming t(8) conditional density for treatment
 #' (W3 <- weightit(re75 ~ age + educ + married +
 #'                   nodegree + re74, data = lalonde,
 #'                 method = "super", density = "dt_8",
 #'                 SL.library = c("SL.glm", "SL.ridge",
 #'                                "SL.glm.interaction")))
-#' summary(W3)
-#' bal.tab(W3)
 #'
-#' #Balancing covariates between treatment groups (binary)
+#' summary(W3)
+#'
+#' cobalt::bal.tab(W3)
+#'
+#' # Balancing covariates between treatment groups (binary)
 #' # using balance SuperLearner to minimize the maximum
 #' # KS statistic
 #' (W4 <- weightit(treat ~ age + educ + married +
@@ -238,15 +254,17 @@
 #'                                "SL.lda"),
 #'                 SL.method = "method.balance",
 #'                 criterion = "ks.max"))
+#'
 #' summary(W4)
-#' bal.tab(W4, stats = c("m", "ks"))}
+#'
+#' cobalt::bal.tab(W4, stats = c("m", "ks"))}
 NULL
 
 weightit2super <- function(covs, treat, s.weights, subset, estimand, focal,
-                           stabilize, subclass, missing, verbose, ...) {
+                           stabilize, missing, verbose, ...) {
 
   covs <- covs[subset, , drop = FALSE]
-  treat <- factor(treat[subset])
+  treat <- treat[subset]
   s.weights <- s.weights[subset]
 
   missing <- .process_missing2(missing, covs)
@@ -262,7 +280,7 @@ weightit2super <- function(covs, treat, s.weights, subset, estimand, focal,
   covs <- as.data.frame(covs)
 
   if (ncol(covs) > 1L) {
-    colinear.covs.to.remove <- colnames(covs)[colnames(covs) %nin% colnames(make_full_rank(covs))]
+    colinear.covs.to.remove <- setdiff(colnames(covs), colnames(make_full_rank(covs)))
     covs <- covs[, colnames(covs) %nin% colinear.covs.to.remove, drop = FALSE]
   }
 
@@ -340,8 +358,9 @@ weightit2super <- function(covs, treat, s.weights, subset, estimand, focal,
                         env = env))
   }, verbose = verbose)},
   error = function(e) {
-    e. <- conditionMessage(e)
-    .err("(from `SuperLearner::SuperLearner()`) ", e., tidy = FALSE)
+    .err(sprintf("(from `SuperLearner::SuperLearner()`): %s",
+                 conditionMessage(e)),
+         tidy = FALSE)
   })
 
   ps <- {
@@ -355,13 +374,14 @@ weightit2super <- function(covs, treat, s.weights, subset, estimand, focal,
   #ps should be matrix of probs for each treat
   #Computing weights
   w <- .get_w_from_ps_internal_bin(ps = ps, treat = treat, estimand,
-                                   stabilize = stabilize, subclass = subclass)
+                                   stabilize = stabilize,
+                                   subclass = ...get("subclass"))
 
   list(w = w, ps = ps, info = info, fit.obj = fit)
 }
 
 weightit2super.multi <- function(covs, treat, s.weights, subset, estimand, focal,
-                                 stabilize, subclass, missing, verbose, ...) {
+                                 stabilize, missing, verbose, ...) {
 
   covs <- covs[subset, , drop = FALSE]
   treat <- factor(treat[subset])
@@ -380,7 +400,7 @@ weightit2super.multi <- function(covs, treat, s.weights, subset, estimand, focal
   covs <- as.data.frame(covs)
 
   if (ncol(covs) > 1L) {
-    colinear.covs.to.remove <- colnames(covs)[colnames(covs) %nin% colnames(make_full_rank(covs))]
+    colinear.covs.to.remove <- setdiff(colnames(covs), colnames(make_full_rank(covs)))
     covs <- covs[, colnames(covs) %nin% colinear.covs.to.remove, drop = FALSE]
   }
 
@@ -421,8 +441,9 @@ weightit2super.multi <- function(covs, treat, s.weights, subset, estimand, focal
                                     env = env))
     }, verbose = verbose)},
     error = function(e) {
-      e. <- conditionMessage(e)
-      .err("(from `SuperLearner::SuperLearner()`) ", e., tidy = FALSE)
+      .err(sprintf("(from `SuperLearner::SuperLearner()`): %s",
+                   conditionMessage(e)),
+           tidy = FALSE)
     })
 
     ps[[i]] <- {
@@ -437,7 +458,8 @@ weightit2super.multi <- function(covs, treat, s.weights, subset, estimand, focal
   #ps should be matrix of probs for each treat
   #Computing weights
   w <- get_w_from_ps(ps = ps, treat = treat, estimand, focal,
-                     stabilize = stabilize, subclass = subclass)
+                     stabilize = stabilize,
+                     subclass = ...get("subclass"))
 
   p.score <- NULL
 
@@ -461,7 +483,7 @@ weightit2super.cont <- function(covs, treat, s.weights, subset, stabilize, missi
   }
 
   if (ncol(covs) > 1L) {
-    colinear.covs.to.remove <- colnames(covs)[colnames(covs) %nin% colnames(make_full_rank(covs))]
+    colinear.covs.to.remove <- setdiff(colnames(covs), colnames(make_full_rank(covs)))
     covs <- covs[, colnames(covs) %nin% colinear.covs.to.remove, drop = FALSE]
   }
 
@@ -534,8 +556,9 @@ weightit2super.cont <- function(covs, treat, s.weights, subset, stabilize, missi
                         env = env))
   }, verbose = verbose)},
   error = function(e) {
-    e. <- conditionMessage(e)
-    .err("(from `SuperLearner::SuperLearner()`) ", e., tidy = FALSE)
+    .err(sprintf("(from `SuperLearner::SuperLearner()`): %s",
+                 conditionMessage(e)),
+         tidy = FALSE)
   })
 
   gp.score <- {
@@ -550,8 +573,8 @@ weightit2super.cont <- function(covs, treat, s.weights, subset, stabilize, missi
   w <- exp(log.dens.num - log.dens.denom)
 
   if (isTRUE(...get("plot"))) {
-    d.n <- attr(log.dens.num, "density")
-    d.d <- attr(log.dens.denom, "density")
+    d.n <- .attr(log.dens.num, "density")
+    d.d <- .attr(log.dens.denom, "density")
     plot_density(d.n, d.d, log = TRUE)
   }
 
@@ -573,8 +596,8 @@ weightit2super.cont <- function(covs, treat, s.weights, subset, stabilize, missi
     # 1) coef: the weights (coefficients) for each algorithm
     # 2) cvRisk: the V-fold CV risk for each algorithm
     computeCoef = function(Z, Y, libraryNames, obsWeights, control, verbose, ...) {
-      estimand <- attr(control$trimLogit, "vals")$estimand
-      init <- attr(control$trimLogit, "vals")$init
+      estimand <- .attr(control$trimLogit, "vals")$estimand
+      init <- .attr(control$trimLogit, "vals")$init
 
       for (i in seq_col(Z)) {
         Z[, i] <- squish(Z[, i], .001)
@@ -634,16 +657,16 @@ weightit2super.cont <- function(covs, treat, s.weights, subset, stabilize, missi
     # 1) coef: the weights (coefficients) for each algorithm
     # 2) cvRisk: the V-fold CV risk for each algorithm
     computeCoef = function(Z, Y, libraryNames, obsWeights, control, verbose, ...) {
-      log.dens.num <- attr(control$trimLogit, "vals")$log.dens.num
-      densfun <- attr(control$trimLogit, "vals")$densfun
-      init <- attr(control$trimLogit, "vals")$init
+      log.dens.num <- .attr(control$trimLogit, "vals")$log.dens.num
+      densfun <- .attr(control$trimLogit, "vals")$densfun
+      init <- .attr(control$trimLogit, "vals")$init
 
-      w_mat <- apply(Z, 2, function(gp.score) {
+      w_mat <- apply(Z, 2L, function(gp.score) {
         r <- Y - gp.score
         exp(log.dens.num - densfun(r / sqrt(col.w.v(r, obsWeights)), log = TRUE))
       })
 
-      cvRisk <- apply(w_mat, 2, cobalt::bal.compute, x = init)
+      cvRisk <- apply(w_mat, 2L, cobalt::bal.compute, x = init)
       names(cvRisk) <- libraryNames
 
       p <- ncol(Z)
