@@ -1,18 +1,12 @@
 #' Weighting methods
 #'
-#' @description `.weightit_methods` is a list containing the allowable weighting
-#'   methods that can be supplied by name to the `method` argument of
-#'   [weightit()], [weightitMSM()], and [weightit.fit()]. Each entry corresponds
-#'   to an allowed method and contains information about what options are and
-#'   are not allowed for each method. While this list is primarily for internal
-#'   use by checking functions in \pkg{WeightIt}, it might be of use for package
-#'   authors that want to support different weighting methods.
+#' @description
+#' `.weightit_methods` is a list containing the allowable weighting methods that can be supplied by name to the `method` argument of [weightit()], [weightitMSM()], and [weightit.fit()]. Each entry corresponds to an allowed method and contains information about what options are and are not allowed for each method. While this list is primarily for internal use by checking functions in \pkg{WeightIt}, it might be of use for package authors that want to support different weighting methods.
 #'
-#' @details Each component is itself a list containing the following components:
+#' @details
+#' Each component is itself a list containing the following components:
 #' \describe{
-#' \item{`treat_type`}{at least one of `"binary"`, `"multi-category"`, or
-#'                     `"continuous"` indicating which treatment types are
-#'                     available for this method.}
+#' \item{`treat_type`}{at least one of `"binary"`, `"multinomial"`, or `"continuous"` indicating which treatment types are available for this method.}
 #' \item{`estimand`}{which estimands are available for this method. All methods that support binary and multi-category treatments accept `"ATE"`, `"ATT"`, and `"ATC"`, as well as some other estimands depending on the method. See [get_w_from_ps()] for more details about what each estimand means.}
 #' \item{`alias`}{a character vector of aliases for the method. When an alias is supplied, the corresponding method will still be dispatched. For example, the canonical method to request entropy balancing is `"ebal"`, but `"ebalance"` and `"entropy"` also work. The first value is the canonical name.}
 #' \item{`description`}{a string containing the description of the name in English.}
@@ -21,6 +15,7 @@
 #' \item{`msm_method_available`}{a logical for whether a version of the method can be used that estimates weights using a single model rather than multiplying the weights across time points. This is related to the `is.MSM.method` argument of `weightitMSM()`.}
 #' \item{`subclass_ok`}{a logical for whether `subclass` can be supplied to compute subclassification weights from the propensity scores.}
 #' \item{`packages_needed`}{a character vector of the minimal packages required to use the method. Some methods may require additional packages for certain options.}
+#' \item{`package_versions_needed`}{a named character vector of the minimal package versions required to use the method. Only present when `packages_needed` is not empty.}
 #' \item{`s.weights_ok`}{a logical for whether sampling weights can be used with the method.}
 #' \item{`missing`}{a character vector of the allowed options that can be supplied to `missing` when missing data is present. All methods accept `"ind"` for the missingness indicator approach; some other methods accept additional values.}
 #' \item{`moments_int_ok`}{a logical for whether `moments`, `int`, and `quantile` can be used with the method.}
@@ -58,7 +53,7 @@
 #' @export
 .weightit_methods <- {list(
   "glm" = list(
-    treat_type = c("binary", "multi-category", "continuous"),
+    treat_type = c("binary", "multinomial", "continuous"),
     estimand = c("ATE", "ATT", "ATC", "ATO", "ATM", "ATOS"),
     alias = c("glm", "ps"),
     description = "propensity score weighting with GLM",
@@ -76,7 +71,7 @@
     plot.weightit_ok = FALSE
   ),
   "bart" = list(
-    treat_type = c("binary", "multi-category", "continuous"),
+    treat_type = c("binary", "multinomial", "continuous"),
     estimand = c("ATE", "ATT", "ATC", "ATO", "ATM", "ATOS"),
     alias = "bart",
     description = "propensity score weighting with BART",
@@ -85,6 +80,7 @@
     msm_method_available = FALSE,
     subclass_ok = TRUE,
     packages_needed = "dbarts",
+    package_versions_needed = c(dbarts = "0.9-29"),
     s.weights_ok = FALSE,
     missing = "ind",
     moments_int_ok = FALSE,
@@ -94,7 +90,7 @@
     plot.weightit_ok = FALSE
   ),
   "cbps" = list(
-    treat_type = c("binary", "multi-category", "continuous"),
+    treat_type = c("binary", "multinomial", "continuous"),
     estimand = c("ATE", "ATT", "ATC", "ATO"),
     alias = c("cbps", "cbgps"),
     description = "covariate balancing propensity score weighting",
@@ -112,7 +108,7 @@
     plot.weightit_ok = FALSE
   ),
   "ebal" = list(
-    treat_type = c("binary", "multi-category", "continuous"),
+    treat_type = c("binary", "multinomial", "continuous"),
     estimand = c("ATE", "ATT", "ATC"),
     alias = c("ebal", "ebalance", "entropy"),
     description = "entropy balancing",
@@ -130,7 +126,7 @@
     plot.weightit_ok = FALSE
   ),
   "energy" = list(
-    treat_type = c("binary", "multi-category", "continuous"),
+    treat_type = c("binary", "multinomial", "continuous"),
     estimand = c("ATE", "ATT", "ATC"),
     alias = c("energy", "dcows"),
     description = "energy balancing",
@@ -148,7 +144,7 @@
     plot.weightit_ok = FALSE
   ),
   "gbm" = list(
-    treat_type = c("binary", "multi-category", "continuous"),
+    treat_type = c("binary", "multinomial", "continuous"),
     estimand = c("ATE", "ATT", "ATC", "ATO", "ATM", "ATOS"),
     alias = c("gbm", "gbr"),
     description = "propensity score weighting with GBM",
@@ -166,7 +162,7 @@
     plot.weightit_ok = TRUE
   ),
   "ipt" = list(
-    treat_type = c("binary", "multi-category"),
+    treat_type = c("binary", "multinomial"),
     estimand = c("ATE", "ATT", "ATC"),
     alias = "ipt",
     description = "inverse probability tilting",
@@ -184,7 +180,7 @@
     plot.weightit_ok = FALSE
   ),
   "npcbps" = list(
-    treat_type = c("binary", "multi-category", "continuous"),
+    treat_type = c("binary", "multinomial", "continuous"),
     estimand = "ATE",
     alias = c("npcbps", "npcbgps"),
     description = "non-parametric covariate balancing propensity score weighting",
@@ -202,16 +198,17 @@
     plot.weightit_ok = FALSE
   ),
   "optweight" = list(
-    treat_type = c("binary", "multi-category", "continuous"),
+    treat_type = c("binary", "multinomial", "continuous"),
     estimand = c("ATE", "ATT", "ATC"),
     alias = c("optweight", "sbw"),
     description = "stable balancing weights",
     ps = FALSE,
     msm_valid = FALSE,
-    msm_method_available = TRUE,
+    msm_method_available = FALSE,
     subclass_ok = FALSE,
-    packages_needed = c("optweight", "osqp"),
-    s.weights_ok = tryCatch(packageVersion("optweight") > "0.2.5", error = function(e) FALSE), #Note; need to update optweight to ensure s.weights are treated same as entropy balancing.
+    packages_needed = "optweight",
+    package_versions_needed = c(optweight = "1.0.0"),
+    s.weights_ok = TRUE,
     missing = "ind",
     moments_int_ok = TRUE,
     moments_default = 1,
@@ -220,7 +217,7 @@
     plot.weightit_ok = TRUE
   ),
   "super" = list(
-    treat_type = c("binary", "multi-category", "continuous"),
+    treat_type = c("binary", "multinomial", "continuous"),
     estimand = c("ATE", "ATT", "ATC", "ATO", "ATM", "ATOS"),
     alias = c("super", "superlearner"),
     description = "propensity score weighting with SuperLearner",
