@@ -35,7 +35,7 @@ test_that("No weights", {
                         data = test_data, family = binomial)
   })
 
-  expect_failure(expect_equal(coef(fit0), coef(fit)))
+  expect_not_equal(coef(fit0), coef(fit), tolerance = eps)
 
   fit_g <- glm(Y_B ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9) + offset(off),
                data = test_data, family = binomial)
@@ -56,7 +56,7 @@ test_that("No weights", {
   fit_g <- glm(Y_B ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
                data = test_data, family = binomial)
   expect_equal(coef(fit), coef(fit_g), tolerance = eps)
-  expect_equal(vcov(fit), sandwich::vcovCL(fit_g, cluster = clus),
+  expect_equal(vcov(fit), sandwich::vcovCL(fit_g, cluster = clus, type = "HC0"),
                tolerance = eps)
 
   #BR
@@ -70,7 +70,34 @@ test_that("No weights", {
                method = brglm2::brglmFit)
 
   expect_equal(coef(fit), coef(fit_g), tolerance = eps)
+  expect_equal(vcov(fit), sandwich::sandwich(fit_g),
+               tolerance = eps * 1e3)
 
+  expect_equal(vcov(fit, cluster = clus),
+               sandwich::vcovCL(fit_g, cluster = clus, type = "HC0"),
+               tolerance = eps * 1e3)
+
+  #Test error for missingness
+  expect_error({
+    fit0 <- glm_weightit(Y_B ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
+                         data = transform(test_data, Y_B = c(NA, Y_B[-1L])),
+                         family = binomial)
+  }, "missing values", ignore.case = TRUE)
+
+  expect_error({
+    fit0 <- glm_weightit(Y_B ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
+                         data = transform(test_data, X1 = c(NA, X1[-1L])),
+                         family = binomial)
+  }, "missing values", ignore.case = TRUE)
+
+  #Test using sandwich functions
+  expect_no_condition({
+    fit0 <- glm_weightit(Y_B ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
+                         data = test_data, family = binomial)
+  })
+
+  expect_equal(vcov(fit0), sandwich::sandwich(fit0),
+               tolerance = eps)
 })
 
 test_that("Binary treatment", {
@@ -94,7 +121,8 @@ test_that("Binary treatment", {
   #M-estimation for glm
   expect_no_condition({
     fit <- glm_weightit(Y_C ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
-                        data = test_data, weightit = W, vcov = "asympt")
+                        data = test_data, weightit = W,
+                        vcov = "asympt")
   })
 
   expect_equal(coef(fit0), coef(fit), tolerance = eps)
@@ -102,37 +130,67 @@ test_that("Binary treatment", {
 
   expect_no_condition({
     fit <- glm_weightit(Y_C ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
-                        data = test_data, weightit = W, vcov = "HC0")
+                        data = test_data, weightit = W,
+                        vcov = "HC0")
   })
 
   expect_equal(coef(fit0), coef(fit), tolerance = eps)
-  expect_failure(expect_equal(vcov(fit0), vcov(fit)))
+  expect_not_equal(vcov(fit0), vcov(fit), tolerance = eps)
 
   set.seed(123)
   expect_no_condition({
     fit <- glm_weightit(Y_C ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
-                        data = test_data, weightit = W, vcov = "FWB", R = 50)
+                        data = test_data, weightit = W,
+                        vcov = "FWB", R = 50)
   })
 
   expect_equal(coef(fit0), coef(fit), tolerance = eps)
-  expect_failure(expect_equal(vcov(fit0), vcov(fit)))
+  expect_not_equal(vcov(fit0), vcov(fit), tolerance = eps)
 
   set.seed(123)
   expect_no_condition({
     fit_ <- glm_weightit(Y_C ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
-                        data = test_data, weightit = W, vcov = "FWB", R = 50,
+                        data = test_data, weightit = W,
+                        vcov = "FWB", R = 50,
                         fwb.args = list(wtype = "mammen"))
   })
 
   expect_equal(coef(fit), coef(fit_), tolerance = eps)
-  expect_failure(expect_equal(vcov(fit), vcov(fit_)))
+  expect_not_equal(vcov(fit), vcov(fit_), tolerance = eps)
 
   expect_no_condition({
     fit <- glm_weightit(Y_C ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
-                        data = test_data, weightit = W, vcov = "BS", R = 50)
+                        data = test_data, weightit = W,
+                        vcov = "BS", R = 50)
   })
 
   expect_equal(coef(fit0), coef(fit), tolerance = eps)
-  expect_failure(expect_equal(vcov(fit0), vcov(fit)))
+  expect_not_equal(vcov(fit0), vcov(fit), tolerance = eps)
 
+  #Test error for missingness
+  expect_error({
+    fit0 <- glm_weightit(Y_B ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
+                         data = transform(test_data, Y_B = c(NA, Y_B[-1L])),
+                         family = binomial)
+  }, "missing values", ignore.case = TRUE)
+
+  expect_error({
+    fit0 <- glm_weightit(Y_B ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
+                         data = transform(test_data, X1 = c(NA, X1[-1L])),
+                         family = binomial)
+  }, "missing values", ignore.case = TRUE)
+
+  #Test using sandwich functions
+  expect_no_condition({
+    fit0 <- glm_weightit(Y_B ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
+                         data = test_data, weightit = W, family = binomial)
+  })
+
+  expect_equal(vcov(fit0),
+               sandwich::sandwich(fit0),
+               tolerance = eps)
+
+  expect_equal(vcov(fit0, type = "HC0"),
+               sandwich::sandwich(fit0, asympt = FALSE),
+               tolerance = eps)
 })
\ No newline at end of file
