@@ -1,4 +1,6 @@
 test_that("vcov arg works in vcov(), summary(), and anova() for glm_weightit", {
+  eps <- if (capabilities("long.double")) 1e-5 else 1e-1
+
   test_data <- readRDS(test_path("fixtures", "test_data.rds"))
   set.seed(123)
   test_data$clus <- sample(1:50, nrow(test_data), replace = TRUE)
@@ -38,79 +40,101 @@ test_that("vcov arg works in vcov(), summary(), and anova() for glm_weightit", {
 
 
   expect_equal(vcov(fit_none, vcov = "asympt"),
-               vcov(fit_asympt))
+               vcov(fit_asympt),
+               tolerance = eps)
 
   expect_equal(vcov(fit_none, vcov = "HC0"),
-               vcov(fit_hc0))
+               vcov(fit_hc0),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(vcov(fit_none, vcov = "BS", R = 25),
-               vcov(fit_bs))
+               vcov(fit_bs),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(vcov(fit_none, vcov = "FWB", R = 25),
-               vcov(fit_fwb))
+               vcov(fit_fwb),
+               tolerance = eps)
 
   expect_equal(vcov(fit_none, vcov = "asympt", cluster = ~clus),
-               vcov(fit_asympt_clus))
+               vcov(fit_asympt_clus),
+               tolerance = eps)
 
   expect_equal(vcov(fit_none, vcov = "HC0", cluster = ~clus),
-               vcov(fit_hc0_clus))
+               vcov(fit_hc0_clus),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(vcov(fit_none, vcov = "BS", R = 25, cluster = ~clus),
-               vcov(fit_bs_clus))
+               vcov(fit_bs_clus),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(vcov(fit_none, vcov = "FWB", R = 25, cluster = ~clus),
-               vcov(fit_fwb_clus))
+               vcov(fit_fwb_clus),
+               tolerance = eps)
 
   expect_equal(vcov(fit_asympt_clus, vcov = "asympt", cluster = NULL),
-               vcov(fit_asympt))
+               vcov(fit_asympt),
+               tolerance = eps)
 
   expect_equal(vcov(fit_asympt_clus, vcov = "asympt"),
-               vcov(fit_asympt_clus))
+               vcov(fit_asympt_clus),
+               tolerance = eps)
 
   expect_equal(vcov(fit_asympt_clus, vcov = "HC0", cluster = NULL),
-               vcov(fit_hc0))
+               vcov(fit_hc0),
+               tolerance = eps)
 
   expect_equal(vcov(fit_asympt_clus, vcov = "HC0"),
-               vcov(fit_hc0_clus))
+               vcov(fit_hc0_clus),
+               tolerance = eps)
 
   expect_equal(summary(fit_asympt, vcov = "HC0")$coef,
-               summary(fit_hc0)$coef)
+               summary(fit_hc0)$coef,
+               tolerance = eps)
 
   expect_equal(summary(fit_asympt, vcov = "HC0", cluster = ~clus)$coef,
-               summary(fit_hc0_clus)$coef)
+               summary(fit_hc0_clus)$coef,
+               tolerance = eps)
 
   expect_equal(summary(fit_asympt_clus, vcov = "HC0", cluster = NULL)$coef,
-               summary(fit_hc0)$coef)
+               summary(fit_hc0)$coef,
+               tolerance = eps)
 
   expect_equal(summary(fit_asympt_clus, vcov = "HC0")$coef,
-               summary(fit_hc0_clus)$coef)
+               summary(fit_hc0_clus)$coef,
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(summary(fit_asympt_clus, vcov = "BS", R = 25)$coef,
-               summary(fit_bs_clus)$coef)
+               summary(fit_bs_clus)$coef,
+               tolerance = eps)
 
   fit_small <- glm_weightit(Y_C ~ A,
                            data = test_data, weightit = W, vcov = "none")
 
   expect_equal(anova(fit_asympt, fit_small),
-               anova(fit_none, fit_small, vcov = "asympt"))
+               anova(fit_none, fit_small, vcov = "asympt"),
+               tolerance = eps)
 
   expect_equal(anova(fit_hc0, fit_small),
-               anova(fit_none, fit_small, vcov = "HC0"))
+               anova(fit_none, fit_small, vcov = "HC0"),
+               tolerance = eps)
 
   expect_equal(anova(fit_asympt_clus, fit_small),
-               anova(fit_none, fit_small, vcov = "asympt", cluster = ~clus))
+               anova(fit_none, fit_small, vcov = "asympt", cluster = ~clus),
+               tolerance = eps)
 
   expect_equal(anova(fit_hc0_clus, fit_small),
-               anova(fit_none, fit_small, vcov = "HC0", cluster = ~clus))
+               anova(fit_none, fit_small, vcov = "HC0", cluster = ~clus),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(anova(fit_bs_clus, fit_small),
-               anova(fit_none, fit_small, vcov = "BS", R = 25, cluster = ~clus))
+               anova(fit_none, fit_small, vcov = "BS", R = 25, cluster = ~clus),
+               tolerance = eps)
 
   expect_error(anova(fit_asympt_clus, fit_small, vcov = "none"),
                "No variance matrix was found")
@@ -126,30 +150,38 @@ test_that("vcov arg works in vcov(), summary(), and anova() for glm_weightit", {
   expect_no_condition(anova(fit_asympt, fit_small_hc0, vcov = "asympt"))
 
   expect_equal(update(fit_none, vcov = "HC0"),
-               fit_hc0)
+               fit_hc0,
+               tolerance = eps)
 
   expect_equal(update(fit_none, vcov = "asympt"),
-               fit_asympt)
+               fit_asympt,
+               tolerance = eps)
 
   expect_equal(update(fit_hc0, vcov = "asympt"),
-               fit_asympt)
+               fit_asympt,
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(fit_bs,
-               update(fit_none, vcov = "BS", R = 25))
+               update(fit_none, vcov = "BS", R = 25),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(fit_fwb,
-               update(fit_none, vcov = "FWB", R = 25))
+               update(fit_none, vcov = "FWB", R = 25),
+               tolerance = eps)
 
   expect_equal(update(fit_hc0, cluster = ~clus),
-               fit_hc0_clus)
+               fit_hc0_clus,
+               tolerance = eps)
 
   expect_equal(update(fit_asympt, cluster = ~clus),
-               fit_asympt_clus)
+               fit_asympt_clus,
+               tolerance = eps)
 
   expect_equal(update(fit_asympt_clus, cluster = NULL),
-               fit_asympt)
+               fit_asympt,
+               tolerance = eps)
 
   #Note: need to remove call because order of arguments is different
   .remove_call <- function(x) {
@@ -159,14 +191,18 @@ test_that("vcov arg works in vcov(), summary(), and anova() for glm_weightit", {
 
   set.seed(123)
   expect_equal(.remove_call(update(fit_bs, R = 25, cluster = ~clus)),
-               .remove_call(fit_bs_clus))
+               .remove_call(fit_bs_clus),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(.remove_call(update(fit_fwb, cluster = ~clus, R = 25)),
-               .remove_call(fit_fwb_clus))
+               .remove_call(fit_fwb_clus),
+               tolerance = eps)
 })
 
 test_that("vcov arg works in vcov(), summary(), and anova() for ordinal_weightit", {
+  eps <- if (capabilities("long.double")) 1e-5 else 1e-1
+
   test_data <- readRDS(test_path("fixtures", "test_data.rds"))
   set.seed(123)
   test_data$clus <- sample(1:50, nrow(test_data), replace = TRUE)
@@ -208,79 +244,101 @@ test_that("vcov arg works in vcov(), summary(), and anova() for ordinal_weightit
 
 
   expect_equal(vcov(fit_none, vcov = "asympt"),
-               vcov(fit_asympt))
+               vcov(fit_asympt),
+               tolerance = eps)
 
   expect_equal(vcov(fit_none, vcov = "HC0"),
-               vcov(fit_hc0))
+               vcov(fit_hc0),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(vcov(fit_none, vcov = "BS", R = 25),
-               vcov(fit_bs))
+               vcov(fit_bs),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(vcov(fit_none, vcov = "FWB", R = 25),
-               vcov(fit_fwb))
+               vcov(fit_fwb),
+               tolerance = eps)
 
   expect_equal(vcov(fit_none, vcov = "asympt", cluster = ~clus),
-               vcov(fit_asympt_clus))
+               vcov(fit_asympt_clus),
+               tolerance = eps)
 
   expect_equal(vcov(fit_none, vcov = "HC0", cluster = ~clus),
-               vcov(fit_hc0_clus))
+               vcov(fit_hc0_clus),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(vcov(fit_none, vcov = "BS", R = 25, cluster = ~clus),
-               vcov(fit_bs_clus))
+               vcov(fit_bs_clus),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(vcov(fit_none, vcov = "FWB", R = 25, cluster = ~clus),
-               vcov(fit_fwb_clus))
+               vcov(fit_fwb_clus),
+               tolerance = eps)
 
   expect_equal(vcov(fit_asympt_clus, vcov = "asympt", cluster = NULL),
-               vcov(fit_asympt))
+               vcov(fit_asympt),
+               tolerance = eps)
 
   expect_equal(vcov(fit_asympt_clus, vcov = "asympt"),
-               vcov(fit_asympt_clus))
+               vcov(fit_asympt_clus),
+               tolerance = eps)
 
   expect_equal(vcov(fit_asympt_clus, vcov = "HC0", cluster = NULL),
-               vcov(fit_hc0))
+               vcov(fit_hc0),
+               tolerance = eps)
 
   expect_equal(vcov(fit_asympt_clus, vcov = "HC0"),
-               vcov(fit_hc0_clus))
+               vcov(fit_hc0_clus),
+               tolerance = eps)
 
   expect_equal(summary(fit_asympt, vcov = "HC0")$coef,
-               summary(fit_hc0)$coef)
+               summary(fit_hc0)$coef,
+               tolerance = eps)
 
   expect_equal(summary(fit_asympt, vcov = "HC0", cluster = ~clus)$coef,
-               summary(fit_hc0_clus)$coef)
+               summary(fit_hc0_clus)$coef,
+               tolerance = eps)
 
   expect_equal(summary(fit_asympt_clus, vcov = "HC0", cluster = NULL)$coef,
-               summary(fit_hc0)$coef)
+               summary(fit_hc0)$coef,
+               tolerance = eps)
 
   expect_equal(summary(fit_asympt_clus, vcov = "HC0")$coef,
-               summary(fit_hc0_clus)$coef)
+               summary(fit_hc0_clus)$coef,
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(summary(fit_asympt_clus, vcov = "BS", R = 25)$coef,
-               summary(fit_bs_clus)$coef)
+               summary(fit_bs_clus)$coef,
+               tolerance = eps)
 
   fit_small <- ordinal_weightit(Y_O ~ A,
                             data = test_data, weightit = W, vcov = "none")
 
   expect_equal(anova(fit_asympt, fit_small),
-               anova(fit_none, fit_small, vcov = "asympt"))
+               anova(fit_none, fit_small, vcov = "asympt"),
+               tolerance = eps)
 
   expect_equal(anova(fit_hc0, fit_small),
-               anova(fit_none, fit_small, vcov = "HC0"))
+               anova(fit_none, fit_small, vcov = "HC0"),
+               tolerance = eps)
 
   expect_equal(anova(fit_asympt_clus, fit_small),
-               anova(fit_none, fit_small, vcov = "asympt", cluster = ~clus))
+               anova(fit_none, fit_small, vcov = "asympt", cluster = ~clus),
+               tolerance = eps)
 
   expect_equal(anova(fit_hc0_clus, fit_small),
-               anova(fit_none, fit_small, vcov = "HC0", cluster = ~clus))
+               anova(fit_none, fit_small, vcov = "HC0", cluster = ~clus),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(anova(fit_bs_clus, fit_small),
-               anova(fit_none, fit_small, vcov = "BS", R = 25, cluster = ~clus))
+               anova(fit_none, fit_small, vcov = "BS", R = 25, cluster = ~clus),
+               tolerance = eps)
 
   expect_error(anova(fit_asympt_clus, fit_small, vcov = "none"),
                "No variance matrix was found")
@@ -296,30 +354,37 @@ test_that("vcov arg works in vcov(), summary(), and anova() for ordinal_weightit
   expect_no_condition(anova(fit_asympt, fit_small_hc0, vcov = "asympt"))
 
   expect_equal(summary(update(fit_none, vcov = "HC0")),
-               summary(fit_hc0))
+               summary(fit_hc0),
+               tolerance = eps)
 
   expect_equal(summary(update(fit_none, vcov = "asympt")),
-               summary(fit_asympt))
+               summary(fit_asympt),
+               tolerance = eps)
 
   expect_equal(update(fit_hc0, vcov = "asympt"),
-               fit_asympt)
+               fit_asympt,
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(fit_bs,
-               update(fit_none, vcov = "BS", R = 25))
+               update(fit_none, vcov = "BS", R = 25),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(fit_fwb,
-               update(fit_none, vcov = "FWB", R = 25))
+               update(fit_none, vcov = "FWB", R = 25),
+               tolerance = eps)
 
   expect_equal(update(fit_hc0, cluster = ~clus),
-               fit_hc0_clus)
+               fit_hc0_clus,
+               tolerance = eps)
 
   expect_equal(update(fit_asympt, cluster = ~clus),
                fit_asympt_clus)
 
   expect_equal(update(fit_asympt_clus, cluster = NULL),
-               fit_asympt)
+               fit_asympt,
+               tolerance = eps)
 
   #Note: need to remove call because order of arguments is different
   .remove_call <- function(x) {
@@ -329,14 +394,18 @@ test_that("vcov arg works in vcov(), summary(), and anova() for ordinal_weightit
 
   set.seed(123)
   expect_equal(.remove_call(update(fit_bs, R = 25, cluster = ~clus)),
-               .remove_call(fit_bs_clus))
+               .remove_call(fit_bs_clus),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(.remove_call(update(fit_fwb, cluster = ~clus, R = 25)),
-               .remove_call(fit_fwb_clus))
+               .remove_call(fit_fwb_clus),
+               tolerance = eps)
 })
 
 test_that("vcov arg works in vcov(), summary(), and anova() for multinom_weightit", {
+  eps <- if (capabilities("long.double")) 1e-5 else 1e-1
+
   test_data <- readRDS(test_path("fixtures", "test_data.rds"))
   test_data$Y_M <- with(test_data, factor(findInterval(Y_C, quantile(Y_C, seq(0, 1, length = 5)),
                                                        all.inside = TRUE)))
@@ -378,79 +447,101 @@ test_that("vcov arg works in vcov(), summary(), and anova() for multinom_weighti
                                    cluster = ~clus)
 
   expect_equal(vcov(fit_none, vcov = "asympt"),
-               vcov(fit_asympt))
+               vcov(fit_asympt),
+               tolerance = eps)
 
   expect_equal(vcov(fit_none, vcov = "HC0"),
-               vcov(fit_hc0))
+               vcov(fit_hc0),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(vcov(fit_none, vcov = "BS", R = 25),
-               vcov(fit_bs))
+               vcov(fit_bs),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(vcov(fit_none, vcov = "FWB", R = 25),
-               vcov(fit_fwb))
+               vcov(fit_fwb),
+               tolerance = eps)
 
   expect_equal(vcov(fit_none, vcov = "asympt", cluster = ~clus),
-               vcov(fit_asympt_clus))
+               vcov(fit_asympt_clus),
+               tolerance = eps)
 
   expect_equal(vcov(fit_none, vcov = "HC0", cluster = ~clus),
-               vcov(fit_hc0_clus))
+               vcov(fit_hc0_clus),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(vcov(fit_none, vcov = "BS", R = 25, cluster = ~clus),
-               vcov(fit_bs_clus))
+               vcov(fit_bs_clus),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(vcov(fit_none, vcov = "FWB", R = 25, cluster = ~clus),
-               vcov(fit_fwb_clus))
+               vcov(fit_fwb_clus),
+               tolerance = eps)
 
   expect_equal(vcov(fit_asympt_clus, vcov = "asympt", cluster = NULL),
-               vcov(fit_asympt))
+               vcov(fit_asympt),
+               tolerance = eps)
 
   expect_equal(vcov(fit_asympt_clus, vcov = "asympt"),
-               vcov(fit_asympt_clus))
+               vcov(fit_asympt_clus),
+               tolerance = eps)
 
   expect_equal(vcov(fit_asympt_clus, vcov = "HC0", cluster = NULL),
-               vcov(fit_hc0))
+               vcov(fit_hc0),
+               tolerance = eps)
 
   expect_equal(vcov(fit_asympt_clus, vcov = "HC0"),
-               vcov(fit_hc0_clus))
+               vcov(fit_hc0_clus),
+               tolerance = eps)
 
   expect_equal(summary(fit_asympt, vcov = "HC0")$coef,
-               summary(fit_hc0)$coef)
+               summary(fit_hc0)$coef,
+               tolerance = eps)
 
   expect_equal(summary(fit_asympt, vcov = "HC0", cluster = ~clus)$coef,
-               summary(fit_hc0_clus)$coef)
+               summary(fit_hc0_clus)$coef,
+               tolerance = eps)
 
   expect_equal(summary(fit_asympt_clus, vcov = "HC0", cluster = NULL)$coef,
-               summary(fit_hc0)$coef)
+               summary(fit_hc0)$coef,
+               tolerance = eps)
 
   expect_equal(summary(fit_asympt_clus, vcov = "HC0")$coef,
-               summary(fit_hc0_clus)$coef)
+               summary(fit_hc0_clus)$coef,
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(summary(fit_asympt_clus, vcov = "BS", R = 25)$coef,
-               summary(fit_bs_clus)$coef)
+               summary(fit_bs_clus)$coef,
+               tolerance = eps)
 
   fit_small <- multinom_weightit(Y_M ~ A,
                                 data = test_data, weightit = W, vcov = "none")
 
   expect_equal(anova(fit_asympt, fit_small),
-               anova(fit_none, fit_small, vcov = "asympt"))
+               anova(fit_none, fit_small, vcov = "asympt"),
+               tolerance = eps)
 
   expect_equal(anova(fit_hc0, fit_small),
-               anova(fit_none, fit_small, vcov = "HC0"))
+               anova(fit_none, fit_small, vcov = "HC0"),
+               tolerance = eps)
 
   expect_equal(anova(fit_asympt_clus, fit_small),
-               anova(fit_none, fit_small, vcov = "asympt", cluster = ~clus))
+               anova(fit_none, fit_small, vcov = "asympt", cluster = ~clus),
+               tolerance = eps)
 
   expect_equal(anova(fit_hc0_clus, fit_small),
-               anova(fit_none, fit_small, vcov = "HC0", cluster = ~clus))
+               anova(fit_none, fit_small, vcov = "HC0", cluster = ~clus),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(anova(fit_bs_clus, fit_small),
-               anova(fit_none, fit_small, vcov = "BS", R = 25, cluster = ~clus))
+               anova(fit_none, fit_small, vcov = "BS", R = 25, cluster = ~clus),
+               tolerance = eps)
 
   expect_error(anova(fit_asympt_clus, fit_small, vcov = "none"),
                "No variance matrix was found")
@@ -466,30 +557,38 @@ test_that("vcov arg works in vcov(), summary(), and anova() for multinom_weighti
   expect_no_condition(anova(fit_asympt, fit_small_hc0, vcov = "asympt"))
 
   expect_equal(summary(update(fit_none, vcov = "HC0")),
-               summary(fit_hc0))
+               summary(fit_hc0),
+               tolerance = eps)
 
   expect_equal(summary(update(fit_none, vcov = "asympt")),
-               summary(fit_asympt))
+               summary(fit_asympt),
+               tolerance = eps)
 
   expect_equal(update(fit_hc0, vcov = "asympt"),
-               fit_asympt)
+               fit_asympt,
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(fit_bs,
-               update(fit_none, vcov = "BS", R = 25))
+               update(fit_none, vcov = "BS", R = 25),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(fit_fwb,
-               update(fit_none, vcov = "FWB", R = 25))
+               update(fit_none, vcov = "FWB", R = 25),
+               tolerance = eps)
 
   expect_equal(update(fit_hc0, cluster = ~clus),
-               fit_hc0_clus)
+               fit_hc0_clus,
+               tolerance = eps)
 
   expect_equal(update(fit_asympt, cluster = ~clus),
-               fit_asympt_clus)
+               fit_asympt_clus,
+               tolerance = eps)
 
   expect_equal(update(fit_asympt_clus, cluster = NULL),
-               fit_asympt)
+               fit_asympt,
+               tolerance = eps)
 
   #Note: need to remove call because order of arguments is different
   .remove_call <- function(x) {
@@ -499,9 +598,11 @@ test_that("vcov arg works in vcov(), summary(), and anova() for multinom_weighti
 
   set.seed(123)
   expect_equal(.remove_call(update(fit_bs, R = 25, cluster = ~clus)),
-               .remove_call(fit_bs_clus))
+               .remove_call(fit_bs_clus),
+               tolerance = eps)
 
   set.seed(123)
   expect_equal(.remove_call(update(fit_fwb, cluster = ~clus, R = 25)),
-               .remove_call(fit_fwb_clus))
+               .remove_call(fit_fwb_clus),
+               tolerance = eps)
 })
\ No newline at end of file
